#ifndef IDEA_H_INCLUDED
#define IDEA_H_INCLUDED
#include "types.h"
#include <stdio.h>
//
// Encrypt/decrypt data[2] using key[4]
//
void IDEA_Encode(u32 *x, u32 const *key);
void IDEA_Decode(u32 *x, u32 const *key);

// Define this to pass test vectors on PC
// (it has to do with endianess)
//
#define IDEA_PC_COMPATIBILITY  

// Optional enforcement against weak keys
// If you enable this, standard test vector set won't pass
//#define IDEA_ENFORCE_AGAINST_WEAK_KEYS

static const u32 IDEA_NUMBER_OF_SUBKEYS = 52;
static const u32 IDEA_ROUNDS = 8;


class IDEA
 {
 u32 encryption_subkeys[IDEA_NUMBER_OF_SUBKEYS];
 u32 decryption_subkeys[IDEA_NUMBER_OF_SUBKEYS];

 public:
    IDEA(u32 const *key = NULL);
    void SetKey(u32 const *key);
    void Encode(u32 *x, u32 const *key);
    void Decode(u32 *x, u32 const *key);
    void Encode(u32 *x);
    void Decode(u32 *x);
    
    void reset(void);
    
 };

int IDEA_Selftest(void);

//----------- Local ---------------------
static void set_encryption_subkeys(u32 *keys, u32 const *k);
static void set_decryption_subkeys(u32 *decryption_subkeys, u32 const *encryption_subkeys);
static u32 invert_subkey(const u32 subkey);
inline static u32 negate_subkey(const u32 subkey);
inline static u32 mod_mul(u32 x, u32 y);
static void idea_process(u32 *x, u32 const* k);

#endif

