#ifndef LARGE_MATH_H_INCLUDED
#define LARGE_MATH_H_INCLUDED

#include "types.h"

// Numbers are least significant dword first

u32 multi_add(u32 *z, u32 const*x, u32 const*y, const u32 length_x32); // Z = X + Y, returns carry (0 or 1)
u32 multi_sub(u32 *z, u32 const*x, u32 const*y, const u32 length_x32); // Z = X + Y, returns carry (0 or 1)
s32 multi_compare(u32 const* x, u32 const *y, const u32 length_x32); // X - Y 

// Note: operands should not overlap with result!
void multi_mul(u32 *z, u32 const*x, u32 const*y, const u32 length_x32); // Z = X * Y
void multi_mul_512(u32 *z, u32 const * x, u32 const * y); // Faster variant for 512 bits

u32 multi_shift_left(u32 *x, const u32 length_x32);
void multi_shift_right(u32 *x, const u32 length_x32);

void multi_shift_left(u32 *x, const u32 value, const u32 length_x32);
void multi_shift_right(u32 *x, const u32 value, const u32 length_x32);

void multi_div(u32 *result,u32 *remainder, u32 *x, u32 *y, const u32 length_x32); // Destroys x,y !


class MULTI_DIV
{
u32 length;
u32 *x;
u32 *y;

public:
    MULTI_DIV(u32 length_x32);
    ~MULTI_DIV();
    void multi_div(u32 *result, u32 *remainder, u32 const *x, u32 const *y);    
    void reset();
};


class SIGNED_MULTI_DIV
{
u32 length;
u32 *x;
u32 *y;

public:
    SIGNED_MULTI_DIV(u32 length_x32);
    ~SIGNED_MULTI_DIV();
    void signed_multi_div(u32 *result, u32 *remainder, u32 const *x, u32 const *y);
    void reset();    

};

class SIGNED_MULTI_MUL
{
u32 length;
u32 *x;
u32 *y;

public:
    SIGNED_MULTI_MUL(u32 length_x32);
    ~SIGNED_MULTI_MUL();
    void signed_multi_mul(u32 *result, u32 const *x, u32 const *y);    
    void reset();    

};


void multi_set_bit(u32 *x, const u32 bit_num);
void multi_clr_bit(u32 *x, const u32 bit_num);
u32 multi_get_bit(u32 const *x, const u32 bit_num);

u32 multi_exp(u32 const *x, const u32 length_x32);

void multi_negate(u32 *x, u32 length_x32); // x = -x





class MULTI_POW_MOD 
{
u32 length;
u32 *pwr_x;
u32 *tmp_prod;
u32 *tmp_div_result;
u32 *tmp_div_reminder;
u32 *tmp_p;

public: 
    MULTI_POW_MOD(u32 length_x32);
    ~MULTI_POW_MOD();
    void multi_pow_mod(u32 *result, const u32 *x, const u32 *y, const u32 *p);   // (x ^ y) modulo p
};

class MODULO_MUL
{
u32 len;    
u32 *acc;
u32 *r;
u32 *q;
u32 *n;

public:
     MODULO_MUL(u32 length_x32);
     ~MODULO_MUL();
     void modulo_mul(u32 *result, u32 const *x, u32 const *y, u32 const* p);

};

u32 equal_to_zero(const u32 *x, const u32 length_x32);
u32 equal_to_one(const u32 *x, const u32 length_x32);
u32 less_then_two(const u32 *x, const u32 length_x32);


u32 multi_inc(u32 *x, const u32 length_x32); // ++
u32 multi_dec(u32 *x, const u32 length_x32); // --


//----------------------------------------------------------------
void extend_2(u32 *extended_x, u32 const * x, const u32 length_x32);
void truncate_2(u32 *x, u32 const *extended_x, const  u32 length_x32);
void copy(u32 *dest, u32 const  *src, const  u32 length_x32);
void copy(u8 *dest, u8 const * src, const u32 length);
void zero(u32 *dest, const  u32 length_x32);

#endif

