// Random numbers 

#include "types.h"
#include "large_math.h"
#include "hash_func.h"
#include "xtea.h"
#include "idea.h"
#include "random.h"
#include "string.h"
#include "system_random.h"
#include <time.h>



// The main seed is 1 KB size

static const u32 seed_size_u32 = 256;

static u32 default_seed[seed_size_u32]; 

u32 *default_seed_ptr(void)
{
return default_seed;
}

static u32 cheap_seed[2];


//--------------------------------------
// Initialize seed

void initialize_seed(void)
{
u32 buff[seed_size_u32];

FILE *file = fopen("seed.bin","rb");

if(file != NULL)
 {
 fread(buff, sizeof(buff), 1, file); 
 fclose(file);
 scramble_seed(default_seed_ptr(), buff, sizeof(buff));
 }

// Wash seed

scramble_random_number(default_seed_ptr());

file = fopen("seed.bin","wb");
fwrite(default_seed_ptr(), 1024, 1, file);
fclose(file);

cheap_random_number(buff, seed_size_u32); 
reset_stack();
}


//----------------------------------
// Collect 64 bit of data entropy into main seed
//
void collect_entropy(u32 const *x)
{
u32 pos = lcg64(cheap_seed) & 0xFE; // even position 0...254

X_64(default_seed + pos, x);
}

void reset_stack(void)
{
const u32 dummy_size = 256;    
u32 dummy[dummy_size];
cheap_random_number(dummy, dummy_size);
}

//---------------------------------

 
void cheap_random_number(u32 *x, const u32 length_x32)
{
for(u32 ci = 0; ci < length_x32; ci++)
 {
 x[ci] = lcg64(cheap_seed);   
 }
}

void cheap_random_number(u8 *x, const u32 size_of_x_bytes)
{
for(u32 ci = 0; ci < size_of_x_bytes; ci++)
 {
 x[ci] = (u8)lcg64(cheap_seed);
 }
}

static inline void mix_round(u32 *seed, u32 x)
{
u32 tmp = seed[0] ^ x;
x += k_mix(seed[0]);
seed[0] = seed[1] ^ x;
seed[1] = tmp;
}
//---------------
// This function can't use 
//
void init_cheap_seed()
{
const u32 time_t_size_x32 = 1 + sizeof(time_t)/sizeof(u32);    

union {    
      time_t t;
      u32 dwords[time_t_size_x32];
      } time_t_dwords;

time(&time_t_dwords.t);

for(u32 ci = 0; ci < 8; ci++)
 {    
 for(u32 cj = 0; cj < time_t_size_x32; cj++) mix_round(cheap_seed, time_t_dwords.dwords[cj]);
 }

cheap_random_number(time_t_dwords.dwords, time_t_size_x32);
reset_stack();
}

//---------------------------
// Simple 64-bit LCG
// Do NOT change since it is used in keystream generation
//
inline u32 lcg64(u32 *state)
{
u64 x = state[0] | (((u64)state[1])<<32);
x = (x << 5) + x + 0x71E86A1CD2D0E1DB;
state[0] = (u32)x;
state[1] = (u32)(x>>32);
return (state[0] ^ state[1]);
}


//--------------------------------------------
// Generation of strong random numbers
// State is u32[256] always
//

void random_number(u32 *x, u32 *state, const u32 length_x32)
{
// Prehash

scramble_random_number(state);

u32 xx[4]; 
 
for(u32 cj = 0; cj < length_x32; cj++)
 { 
 lcg64(state);

 xx[0] = state[0];
 xx[1] = state[1];
 xx[2] = state[2];
 xx[3] = state[seed_size_u32/2 - 1]; 

 X_64(xx, xx + 2);
 
 u32 xtmp = xx[0]; 

 for(u32 ci = 3; ci < seed_size_u32; ci++)
  {
  state[ci-1] = state[ci] ^ xtmp;
  xtmp += k_mix(state[ci]);
  }
 
 state[seed_size_u32 - 1] = xx[1];
 x[cj] = xtmp;
 } // cj

// Posthash

cheap_random_number(xx, 4);
scramble_random_number(state);
reset_stack();
}
//-------------------------------------
// Scramble 1kbyte seed u32[256] with some external data
//

void scramble_seed(u32 *seed, void const *data, const u32 data_size_bytes)
{
u32 rounds = 3 + data_size_bytes/(sizeof(u32)*seed_size_u32);
u8 const *ptr = (u8 const *)data;
u32 tmp_data[2] = { 0, 0 };
u32 cntr = data_size_bytes;
u32 idx = 0;

for(u32 ci = 0; ci < rounds; ci++)
 {
 for(u32 cj = 0; cj < seed_size_u32; cj += 2)
  {

  for(u32 ck = 0; ck < 8; ck++)
   {
   tmp_data[1] = (tmp_data[1] << 8) | (tmp_data[0] >> 24);
   tmp_data[0] = (tmp_data[0] << 8) | (u32)(ptr[idx]);
   tmp_data[0] += cntr;  // tweak value
   cntr++; 
   idx++;
   if(idx >= data_size_bytes) idx = 0;
   }

  if(!cj) 
   {
   seed[cj + 0] += seed[seed_size_u32 - 2];
   seed[cj + 1] += seed[seed_size_u32 - 1]; 
   }
  else
   {
   seed[cj + 0] += seed[cj - 2];
   seed[cj + 1] += seed[cj - 1];
   }
 
  X_64(seed + cj, tmp_data);   
  }
 }
cheap_random_number(tmp_data, 2);
reset_stack();
}

#ifndef _WINDOWS
#include <fcntl.h>
#include <unistd.h>
int Get_System_Provided_Entropy(u8 *buffer, u32 buffer_size_bytes)
{
 int randomData = open("/dev/urandom", O_RDONLY);
 int size = read(randomData, (char*)buffer, buffer_size_bytes);
 close(randomData);
 return size;
}
#endif

void scramble_random_number(u32 *state)
{
const u32 foo_buffer_size_bytes = 1024;
u32 foo_buffer[foo_buffer_size_bytes/sizeof(u32)];

Get_System_Provided_Entropy((u8 *)foo_buffer, foo_buffer_size_bytes);

scramble_seed(state, foo_buffer, foo_buffer_size_bytes);

time_t t;

time(&t);

scramble_seed(state, &t, sizeof(time_t));

init_cheap_seed();
reset_stack();
cheap_random_number(foo_buffer, foo_buffer_size_bytes/sizeof(u32));
}

//------------------------------------------------
// State is [k0].....k[11]
// 
// state u32[12] = 384 bit state
// state[0..1] is LCG_A
// state[2..3] is LCG_B
// state[4..7] is key_A
// state[8..11] is key_B
//
// Generates u32[2] pseudorandom number 
//
static void keystream_generator(u32 *output, u32 *state)
{
// Irreversible function
// Even if the attacker would recover current state of CPRNG,
// It won't be possible to decode previous information 


// Output encode

lcg64(state + 0);

output[0] = state[0];
output[1] = state[1];

IDEA_Encode(output, state + 4); // No need to store IDEA round keys

// Hash encode

lcg64(state + 2);
X_128(state + 8, state + 2);

// Modify key

state[4] += state[8];
state[5] += state[9];
state[6] += state[10];
state[7] += state[11];
}

//===============
// Initialize keystream state u32[12] with key u32 [8]
//
static void keystream_generator_init(u32 *state, u32 const *key)
{

// Use 256 bits of key as is

copy(state, key, 8);

// Compute 128 bit hash to expand key

state[8] = key[3];
state[9] = key[2];
state[10] = key[1];
state[11] = key[0];

for(u32 ci = 0; ci < 8; ci += 2)
 {
 X_128(state + 8, key + ci);
 }

reset_stack();
}
//---------------------------------------------

KEYSTREAM_GENERATOR::KEYSTREAM_GENERATOR()
{
KEYSTREAM_GENERATOR::reset();
}

KEYSTREAM_GENERATOR::~KEYSTREAM_GENERATOR()
{
KEYSTREAM_GENERATOR::reset(); 
}

void KEYSTREAM_GENERATOR::reset()
{
cntr = 0;
byte_buffer_idx = 0;
memset(byte_buffer,0, sizeof(byte_buffer));
cheap_random_number(state, 12);
reset_stack();
}

void KEYSTREAM_GENERATOR::init_key(u32 const* key)
{
keystream_generator_init(state, key);
cntr = 0;
byte_buffer_idx = 0;
}

u8 KEYSTREAM_GENERATOR::get_keystream_byte()
{

if(!byte_buffer_idx) // time to generate next 8 bytes
 {
 u32 tmp_buf[2];   
 keystream_generator(tmp_buf, state);
 
 for(u32 ci = 0; ci < 4; ci++)
  {
  byte_buffer[ci]   = (u8)(tmp_buf[0] & 0xFF);
  tmp_buf[0] >>= 8;

  byte_buffer[ci+4] = (u8)(tmp_buf[1] & 0xFF);  
  tmp_buf[1] >>= 8;
  }
 }

u8 ret_value = byte_buffer[byte_buffer_idx];
byte_buffer_idx = (byte_buffer_idx + 1) & 0x07;

cntr++;


return ret_value;
}

int KEYSTREAM_GENERATOR::counter(u64 x)
{
if(x < cntr) return 0; // Can't go back in time

while(x > cntr) get_keystream_byte(); // Roll forward

return 1;
}

u64 KEYSTREAM_GENERATOR::counter()
{
return cntr;
}

//==================== <END> ========================

