#ifndef RANDOM_H_INCLUDED
#define RANDOM_H_INCLUDED
#include "types.h"

// State is u32 seed[256]; = 1 kilobyte of state

static const u32 size_of_seed_bytes = 1024;

void random_number(u32 *x, u32 *state, const u32 length_x32);
void scramble_random_number(u32 *state);
//----------------------------------
// Collect entropy into main seed
//
void collect_entropy(u32 const * x);
void reset_stack(void);
void cheap_random_number(u32 *x, const u32 length_x32);
void cheap_random_number(u8 *x, const u32 size_of_x_bytes);

void init_cheap_seed();

void initialize_seed(void); // Initialize main seed file
void scramble_seed(u32 *seed, void const *data, const u32 data_size_bytes);

inline u32 lcg64(u32 *state);
u32 *default_seed_ptr(void);


class KEYSTREAM_GENERATOR
{
u8 byte_buffer[8];
u32 byte_buffer_idx;
u32 state[12];
u64 cntr;

public:
    KEYSTREAM_GENERATOR();
    ~KEYSTREAM_GENERATOR();
    
    void init_key(u32 const* key); // Initalize with 256-bit key
    u8 get_keystream_byte(); // Generate pseudorandom byte
    int counter(u64 cnt); // returns 0 if counter less then current (can't go back in time)
    u64 counter(); // returns counter
    void reset();  

};


static void keystream_generator(u32 *output, u32 *state); // generate u32[2] pseudorandom numbers 
static void keystream_generator_init(u32 *state, u32 const* key); // initialise state u32[12] with key u32[8]
static inline void mix_round(u32 *seed, u32 x);


#endif

