#ifndef SIMPLE_TEST_H_INCLUDED
#define SIMPLE_TEST_H_INCLUDED

#include "types.h"
#include "large_math.h"
#include "fast_modulo.h"
#include "montgomery.h"
#include "fast_power_2.h"

//------------------------------------------------------
// test if g is a prime root for P
// (it is asummed that P is safe prime, i.e. (P - 1)/2 is also prime
//  so, p-1 has two dividers: q = 2 and q = (P-1)/2
// 
// g is generator if g^(p-1)/q mod P != 1
// 

class TEST_GENERATOR_MODULO_P
{
u32 len;
MONTGOMERY_POW_MOD *mpm;
u32 *q;
u32 *result;
u32 *y;

public:
    TEST_GENERATOR_MODULO_P(u32 length_x32);
    ~TEST_GENERATOR_MODULO_P();
    int test_generator_modulo_p(u32 const *g, u32 const *p); 
    void reset();   
};


typedef struct 
{
u32 *divident;
u32 *divisor;
u32 *quotient;
u32 *remainder;

u32 *m;
u32 *a;
u32 *z;
u32 *p1;

u32 *z2;

MONTGOMERY_POW_MOD *mpm;
FAST_POW_MOD_2 *fast_pow_mod_2;

} MILLER_RABIN_MEMORY;


// Sieve for safe prime P from given base to given sieve length

int Sieve_Iteration(u32 *base, const u32 length_x32, u32 sieve_length, const u32 mrt_rounds);

int Strong_Prime_Test(u32 const *x, const u32 rounds, const u32 length_x32);
int MRT_core(u32 const *x, MILLER_RABIN_MEMORY *mrm, const u32 length_x32, u32 small_witness_param = 0);
void Prepare_Prime_For_Test(u32 *x, u32 length_x32);

MILLER_RABIN_MEMORY * Miller_Rabin_Init(const u32 length_x32);
void Miller_Rabin_UnInit(MILLER_RABIN_MEMORY * mrm);

//------------ Local --------

static int MRT_2_small_core(u32 const *x, MILLER_RABIN_MEMORY *mrm, const u32 length_x32); // MRT using 2 as witness
static u32 count_right_side_zeroes(u32 *m, u32 const *x, const u32 length_x32);



#endif

