#include "types.h"

// XTEA (c) 1997 . Roger M. Needham and David J. Wheeler.
// 64-bit block (u32 v[2]) is encoded/decoded using 128-bit key (u32 k[4])  
//

static const u32 xtea_rounds = 33;  // XTEA rounds (33 used to achieve full diffusion)
static const u32 delta = 0x9E3779B9;  // XTEA internal value


//----------------------------------------------------

void XTEA_Encode(u32* v, u32 const* k) 
{
u32 sum = 0;
u32 v0 = v[0];
u32 v1 = v[1];
u16 rounds = xtea_rounds;   

while(rounds--) 
 {
 v0  += (((v1 << 4) ^ (v1 >> 5)) + v1) ^ (sum + k[sum&3]);
 sum += delta;
 v1  += (((v0 << 4) ^ (v0 >> 5)) + v0) ^ (sum + k[(sum>>11)&3]);
 }

v[0] = v0; 
v[1] = v1;
}
//-------------

void XTEA_Decode(u32* v, u32 const* k) 
{
u32 sum=(u32)(((u64)delta)*xtea_rounds); // To avoid warning on u32 overflow
u32 v0 = v[0];
u32 v1 = v[1];
u16 rounds = xtea_rounds;

while(rounds--) 
 {
 v1  -= (((v0 << 4) ^ (v0 >> 5)) + v0) ^ (sum + k[(sum>>11) & 3]);
 sum -= delta;
 v0 -= (((v1 << 4) ^ (v1 >> 5)) + v1) ^ (sum + k[sum & 3]);
 }

v[0] = v0; 
v[1] = v1;
}
//----------------------------------
// XTEA from original article
//
void tean(long * v, long * k, long N) 
{
unsigned long y=v[0], z=v[1], DELTA=0x9e3779b9;

if(N>0) 
{
// coding 
unsigned long limit=DELTA*N, sum=0 ;
while(sum!=limit)
y+= (z<<4 ^ z>>5) + z ^ sum + k[sum&3],
sum+=DELTA,
z+= (y<<4 ^ y>>5) + y ^ sum + k[sum>>11 &3] ;
}
else 
{
// decoding
unsigned long sum=DELTA*(-N) ;
while (sum)
z-= (y<<4 ^ y>>5) + y ^ sum + k[sum>>11 &3],
sum-=DELTA,
y-= (z<<4 ^ z>>5) + z ^ sum + k[sum&3] ;
}
v[0]=y, v[1]=z;
}



//================= <END> ==============
