#include <windows.h>
#include <commctrl.h>
#include <Windowsx.h>
#include <Windowsx.h>
#include "lists.h"
#include "main.h"
#include "resource.h"
//----------------------------------------------------------------------------
HWND g_NetList = NULL;
HWND g_KadList = NULL;
//----------------------------------------------------------------------------
static void WndSizing(HWND hWnd, WPARAM wParam, LPRECT newWndRect)
{
	RECT oldWndRect;
	RECT oldClientRect;
	const LONG lMinWidth = 400;
	const LONG lMinHeight = 300;
	//
	GetWindowRect(hWnd, &oldWndRect);
	GetClientRect(hWnd, &oldClientRect);
	//
	oldClientRect.right -= oldWndRect.right - oldWndRect.left - (newWndRect->right - newWndRect->left);		// calculate new client width
	oldClientRect.bottom -= oldWndRect.bottom - oldWndRect.top - (newWndRect->bottom - newWndRect->top);	// calculate new client height
	// adjust width
	if (oldClientRect.right < lMinWidth){
		switch (wParam)
		{
		case WMSZ_BOTTOMLEFT:
		case WMSZ_LEFT:
		case WMSZ_TOPLEFT:
			newWndRect->left -= lMinWidth - oldClientRect.right;
			break;
			//
		default:
			newWndRect->right += lMinWidth - oldClientRect.right;
		}
	}
	// adjust height
	if (oldClientRect.bottom < lMinHeight){
		switch (wParam)
		{
		case WMSZ_TOPLEFT:
		case WMSZ_TOP:
		case WMSZ_TOPRIGHT:
			newWndRect->top -= lMinHeight - oldClientRect.bottom;
			break;
			//
		default:
			newWndRect->bottom += lMinHeight - oldClientRect.bottom;
		}
	}
}
//----------------------------------------------------------------------------
static void WndSize(HWND hWnd)
{
	RECT rc;
	GetClientRect(hWnd,&rc);

	MoveWindow(GetDlgItem(hWnd,IDC_USERS),0,0,rc.right,rc.bottom,TRUE);
}
//----------------------------------------------------------------------------
static void UpdateNetList(HWND hWnd)
{
	TCHAR str[100];
	HWND hUsers = GetDlgItem(hWnd,IDC_USERS);
	NETWORK* net = getNet();
    CONNECTION* ptr;
    int cnt = 0;
	int cur_sel = ListView_GetSelectionMark(hUsers);
	u32 sel_id[NETWORK_ID_LENGTH];
	int num = ListView_GetItemCount(hUsers);
	if (!net) return;
	if (!ListView_GetSelectedCount(hUsers))
	{
		cur_sel = -1;
	}
	if (cur_sel >= 0)
	{
		ListView_GetItemText(hUsers,cur_sel,3,str,100);
		str2id(str,sel_id);
	}
	for(ptr = net->get_first_node(); ptr; ptr = net->get_next_node(ptr))
	{
		LVITEM item = {0};
			
		item.mask = LVIF_TEXT|LVIF_IMAGE;
		item.pszText = (LPTSTR)ptr->userInfo;
		item.iImage = isTrustUser(ptr->nd.id)?1:0;
		item.iItem = cnt++;
		if (num < cnt) item.iItem = ListView_InsertItem(hUsers,&item);
		else ListView_SetItem(hUsers,&item);
		item.mask = LVIF_TEXT;
		item.iSubItem = 1;
		item.pszText = str;
		ip2str(ptr->nd.ip,str);
		ListView_SetItem(hUsers,&item);
		item.iSubItem = 2;
		_stprintf_s(str,100,TEXT("%d"),ptr->nd.port);
		ListView_SetItem(hUsers,&item);
		item.iSubItem = 3;
		id2str(ptr->nd.id,str);
		ListView_SetItem(hUsers,&item);

		if ((cur_sel >= 0) && isIdIdentical(sel_id,ptr->nd.id))
		{
			ListView_SetItemState(hUsers,item.iItem,LVIS_SELECTED,LVIS_SELECTED);
			ListView_SetSelectionMark(hUsers,item.iItem);
		}
	}
	// clear all other items if exist
	while(cnt < (num = ListView_GetItemCount(hUsers)))
	{
		ListView_DeleteItem(hUsers,num-1);
	}
	ListView_RedrawItems(hUsers,0,cnt);
}
//----------------------------------------------------------------------------
static void ListInit(HWND hWnd)
{
	LVITEM item = {0};
	LVCOLUMN col;
	HWND hUsers = GetDlgItem(hWnd,IDC_USERS);
	ListView_SetExtendedListViewStyle(hUsers,WS_EX_CLIENTEDGE|LVS_EX_FULLROWSELECT|LVS_EX_AUTOSIZECOLUMNS|LVS_EX_GRIDLINES);
	// create columns
	col.mask = LVCF_WIDTH|LVCF_TEXT;
	col.pszText = TEXT("Name");
	col.cx = 100;
	ListView_InsertColumn(hUsers,0,&col);
	col.pszText = TEXT("IP");
	ListView_InsertColumn(hUsers,1,&col);
	col.pszText = TEXT("Port");
	ListView_InsertColumn(hUsers,2,&col);
	col.pszText = TEXT("ID");
	col.cx = 500;
	ListView_InsertColumn(hUsers,3,&col);
	// add image list
    HIMAGELIST hLarge = ImageList_Create(GetSystemMetrics(SM_CXICON),GetSystemMetrics(SM_CYICON),ILC_COLOR32,2,1);
    HIMAGELIST hSmall = ImageList_Create(GetSystemMetrics(SM_CXSMICON),GetSystemMetrics(SM_CYSMICON),ILC_COLOR32, 2, 1);
    HICON hiconItem = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_GREY_USER));
    ImageList_AddIcon(hLarge, hiconItem);
    ImageList_AddIcon(hSmall, hiconItem);
    DestroyIcon(hiconItem);
    hiconItem = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_GREEN_USER));
    ImageList_AddIcon(hLarge, hiconItem);
    ImageList_AddIcon(hSmall, hiconItem);
    DestroyIcon(hiconItem);
	ListView_SetImageList(hUsers,hLarge,LVSIL_NORMAL);
	ListView_SetImageList(hUsers,hSmall,LVSIL_SMALL);
	SetTimer(hWnd,1,1000,NULL);
	WndSize(hWnd);
}
//----------------------------------------------------------------------------
static void ListDestroy(HWND hWnd)
{
	KillTimer(hWnd,1);
}
//----------------------------------------------------------------------------
static void UpdateKadList(HWND hWnd)
{
	TCHAR str[100];
	HWND hUsers = GetDlgItem(hWnd,IDC_USERS);
	KADEMLIA* kad = getKad();
	NETWORK* net = getNet();
    DHT* ptr;
    int cnt = 0;
	int cur_sel = ListView_GetSelectionMark(hUsers);
	u32 sel_id[NETWORK_ID_LENGTH];
	TCHAR user_info[user_info_max_length_bytes/sizeof(TCHAR)];
	int num = ListView_GetItemCount(hUsers);
	if (!kad) return;
	if (!ListView_GetSelectedCount(hUsers))
	{
		cur_sel = -1;
	}
	if (cur_sel >= 0)
	{
		ListView_GetItemText(hUsers,cur_sel,3,str,100);
		str2id(str,sel_id);
	}
	for (u32 buck = 0; buck < NETWORK_ID_LENGTH*32; buck++)
	{
		LVITEM item;
		ZeroMemory(&item,sizeof(LVITEM));
		item.mask = LVIF_TEXT|LVIF_IMAGE;
		item.pszText = str;
		item.iImage = -1;
		item.iItem = cnt++;
		_stprintf_s(str,100,TEXT("Group %d"),buck);
		if (num < cnt) item.iItem = ListView_InsertItem(hUsers,&item);
		else ListView_SetItem(hUsers,&item);
		item.mask = LVIF_TEXT;
		item.iSubItem = 1;
		item.pszText = TEXT("");
		ListView_SetItem(hUsers,&item);
		item.iSubItem = 2;
		ListView_SetItem(hUsers,&item);
		item.iSubItem = 3;
		ListView_SetItem(hUsers,&item);
		
		for(ptr = kad->get_first_node(buck); ptr; ptr = kad->get_next_node(ptr))
		{
			item.mask = LVIF_TEXT|LVIF_IMAGE;
			net->get_user_info(&ptr->node,user_info);
			item.pszText = user_info;
			item.iImage = isTrustUser(ptr->node.id)?1:0;
			item.iItem = cnt++;
			item.iSubItem = 0;
			if (num < cnt) item.iItem = ListView_InsertItem(hUsers,&item);
			else ListView_SetItem(hUsers,&item);
			item.mask = LVIF_TEXT;
			item.iSubItem = 1;
			item.pszText = str;
			ip2str(ptr->node.ip,str);
			ListView_SetItem(hUsers,&item);
			item.iSubItem = 2;
			_stprintf_s(str,100,TEXT("%d"),ptr->node.port);
			ListView_SetItem(hUsers,&item);
			item.iSubItem = 3;
			id2str(ptr->node.id,str);
			ListView_SetItem(hUsers,&item);

			if ((cur_sel >= 0) && isIdIdentical(sel_id,ptr->node.id))
			{
				ListView_SetItemState(hUsers,item.iItem,LVIS_SELECTED,LVIS_SELECTED);
				ListView_SetSelectionMark(hUsers,item.iItem);
			}
		}
	}
	// clear all other items if exist
	while(cnt < (num = ListView_GetItemCount(hUsers)))
	{
		ListView_DeleteItem(hUsers,num-1);
	}
	ListView_RedrawItems(hUsers,0,cnt);
}
//----------------------------------------------------------------------------
static void ActivateChat(HWND hWnd, int iItem)
{
	TCHAR str[100];
	u32 id[NETWORK_ID_LENGTH];
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),iItem,3,str,100);
	if (str2id(str,id))
	{
		StartChat(id);
	}
}
//----------------------------------------------------------------------------
static void userMenu(HWND hWnd,LPNMITEMACTIVATE ntf)
{
	NODE nd;
	TCHAR str[100]=TEXT("");
	HMENU menu = CreatePopupMenu();
	MENUITEMINFO item;
	TCHAR UserName[200];

	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),ntf->iItem,3,str,100);
	if (!str2id(str,nd.id)) return;
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),ntf->iItem,0,UserName,200);
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),ntf->iItem,1,str,100);
	str2ip(str,&nd.ip);
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),ntf->iItem,2,str,100);
	nd.port = (u16)wcstol(str,NULL,10);

	item.cbSize = sizeof(MENUITEMINFO);
	item.fMask = MIIM_TYPE|MIIM_ID;
	item.fType = MFT_STRING;
	if (isTrustUser(nd.id))
	{
		item.wID = IDM_REMOVETRUST;
		item.dwTypeData = TEXT("Remove from trust list");
	}
	else
	{
		item.wID = IDM_ADDTRUST;
		item.dwTypeData = TEXT("Add to trust list");
	}
	InsertMenuItem(menu,0,TRUE,&item);
	item.wID = IDM_ADDSAVED;
	item.dwTypeData = TEXT("Add to saved list");
	InsertMenuItem(menu,1,TRUE,&item);
	ClientToScreen(GetDlgItem(hWnd,IDC_USERS),&ntf->ptAction);
	switch (TrackPopupMenu(menu,TPM_RETURNCMD,ntf->ptAction.x,ntf->ptAction.y,0,hWnd,NULL))
	{
	case IDM_ADDTRUST:
		AddTrustId(nd.id);
		break;
	case IDM_REMOVETRUST:
		RemoveTrustId(nd.id);
		break;
	case IDM_ADDSAVED:
		appendLike(&nd,UserName);
		break;
	}
}
//----------------------------------------------------------------------------
INT_PTR CALLBACK NetListProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG: ListInit(hWnd); return TRUE;
    case WM_DESTROY: ListDestroy(hWnd); return 0;
	case WM_SIZE: WndSize(hWnd); return 0;
	case WM_SIZING: WndSizing(hWnd,wParam,(LPRECT)lParam); return TRUE;
	case WM_CLOSE: ShowWindow(hWnd,SW_HIDE); return FALSE;
	case WM_TIMER: UpdateNetList(hWnd);
	case WM_NOTIFY:
	{
		LPNMITEMACTIVATE ntf = (LPNMITEMACTIVATE)lParam;
		if (ntf && ntf->hdr.idFrom == IDC_USERS)
		{
			switch (ntf->hdr.code)
			{
			case NM_DBLCLK:
				ActivateChat(hWnd,ntf->iItem);
				break;
			case NM_RCLICK:
				userMenu(hWnd,ntf);
				break;
			}
		}
		return FALSE;
	}
	}
	return FALSE;
}
//----------------------------------------------------------------------------
INT_PTR CALLBACK KadListProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG: ListInit(hWnd); return TRUE;
    case WM_DESTROY: ListDestroy(hWnd); return 0;
	case WM_SIZE: WndSize(hWnd); return 0;
	case WM_SIZING: WndSizing(hWnd, wParam, (LPRECT)lParam); return TRUE;
	case WM_CLOSE: ShowWindow(hWnd,SW_HIDE); return FALSE;
	case WM_TIMER: UpdateKadList(hWnd);
	case WM_NOTIFY:
	{
		LPNMITEMACTIVATE ntf = (LPNMITEMACTIVATE)lParam;
		if (ntf && ntf->hdr.idFrom == IDC_USERS)
		{
			switch (ntf->hdr.code)
			{
			case NM_DBLCLK:
				ActivateChat(hWnd,ntf->iItem);
				break;
			case NM_RCLICK:
				userMenu(hWnd,ntf);
				break;
			}
		}
		return FALSE;
	}
	}
	return FALSE;
}
//----------------------------------------------------------------------------
void CreateListsDialog(HWND hWnd)
{
	g_NetList = CreateDialog(GetModuleHandle(NULL),TEXT("Network"),hWnd,NetListProc);
	g_KadList = CreateDialog(GetModuleHandle(NULL),TEXT("Kademlia"),hWnd,KadListProc);
}
//----------------------------------------------------------------------------
