#include <windows.h>
#include <Windowsx.h>
#include <commctrl.h>
#include <tchar.h>
#include <stdio.h>

#include "resource.h"
#include "settings.h"
//----------------------------------------------------------------------------
HWND g_logWnd = NULL;
//----------------------------------------------------------------------------
typedef struct {
	FILE* F;
	CRITICAL_SECTION cs;
} params_t;
//----------------------------------------------------------------------------
static void LogSize(HWND hWnd)
{
	RECT rc;
	GetClientRect(hWnd,&rc);
	MoveWindow(GetDlgItem(hWnd,IDC_LOG),0,0,rc.right,rc.bottom,TRUE);
}
//----------------------------------------------------------------------------
static void ApplySettings(HWND hWnd,settings_t* set)
{
	params_t *par = (params_t*)GetProp(hWnd,TEXT("par"));
	EnterCriticalSection(&par->cs);
	if (par->F) {
		fclose(par->F);
		par->F = NULL;
	}
	if (set->enableLogFile && _tfopen_s(&par->F,set->logFileName,TEXT("w")))
	{
		MessageBox(hWnd,TEXT("Can't create log file"),TEXT("Worning"),MB_OK|MB_ICONWARNING);
	}
	LeaveCriticalSection(&par->cs);
}
//----------------------------------------------------------------------------
static void LogInit(HWND hWnd, settings_t* set)
{
	LVCOLUMN col;
	params_t *par = (params_t*)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(params_t));
	SetProp(hWnd,TEXT("par"),par);
	HWND hLog = GetDlgItem(hWnd,IDC_LOG);
	ListView_SetExtendedListViewStyle(hLog,LVS_EX_FULLROWSELECT|/*LVS_EX_AUTOSIZECOLUMNS|*/LVS_EX_GRIDLINES);
	// create columns
	col.mask = LVCF_WIDTH|LVCF_TEXT;
	col.cx = 150;
	col.pszText = TEXT("Time");
	ListView_InsertColumn(hLog,0,&col);
	col.cx = 100;
	col.pszText = TEXT("Source");
	ListView_InsertColumn(hLog,1,&col);
	col.cx = 800;
	col.pszText = TEXT("Message");
	ListView_InsertColumn(hLog,2,&col);
	g_logWnd = hWnd;
	InitializeCriticalSection(&par->cs);
	ApplySettings(hWnd,set);
	LogSize(hWnd);
}
//----------------------------------------------------------------------------
static void LogDestroy(HWND hWnd)
{
	params_t *par = (params_t*)RemoveProp(hWnd,TEXT("par"));
	DeleteCriticalSection(&par->cs);
	if (par->F) fclose(par->F);
	HeapFree(GetProcessHeap(),0,par);
}
//----------------------------------------------------------------------------
INT_PTR CALLBACK LogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		LogInit(hWnd,(settings_t*)lParam);
		return TRUE;
		//
    case WM_DESTROY:
		LogDestroy(hWnd);
		return 0;
		//
	case WM_CLOSE:
		ShowWindow(hWnd,SW_HIDE);
		return TRUE;
		//
	case WM_SIZE:
		LogSize(hWnd);
		return TRUE;
		//
	case WM_APPLY_LOG_SETTINGS:
		ApplySettings(hWnd,(settings_t*)lParam);
		return TRUE;
	}
	return FALSE;
}
//-----------------------------------------------------------------------
void LogMessage(LPTSTR module, LPTSTR message)
{
	TCHAR datetime[100];
	SYSTEMTIME time;
	LVITEM item = {0};
	params_t *par = (params_t*)GetProp(g_logWnd,TEXT("par"));

	GetLocalTime(&time);
	_stprintf_s(datetime,100,TEXT("%02d/%02d/%02d %02d:%02d:%02d"),time.wYear%100,time.wMonth,time.wDay,time.wHour,time.wMinute,time.wSecond);

	item.mask = LVIF_TEXT;
	item.pszText = datetime;
	item.iItem = ListView_GetItemCount(GetDlgItem(g_logWnd,IDC_LOG));
	item.iItem = ListView_InsertItem(GetDlgItem(g_logWnd,IDC_LOG),&item);
	item.iSubItem = 1;
	item.pszText = module;
	ListView_SetItem(GetDlgItem(g_logWnd,IDC_LOG),&item);
	item.iSubItem = 2;
	item.pszText = message;
	ListView_SetItem(GetDlgItem(g_logWnd,IDC_LOG),&item);

	if (par->F)
	{
		EnterCriticalSection(&par->cs);
		_ftprintf_s(par->F,TEXT("%s - %s:\t%s\n"),datetime,module,message);
		LeaveCriticalSection(&par->cs);
	}
	
	if (par->F) fflush(par->F);
}
//-----------------------------------------------------------------------
