#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <Windowsx.h>

#include "resource.h"
#include "settings.h"
#include "password.h"
#include "network.h"
#include "kademlia.h"
#include "chat.h"
#include "build.h"
#include "upnp.h"
#include "log.h"
#include "users.h"
#include "lists.h"
#include "..\audio\waveio.h"
//----------------------------------------------------------------------------
static LPTSTR module = TEXT("Main");
//----------------------------------------------------------------------------
typedef struct {
	HWND hWnd;
	u32 id[NETWORK_ID_LENGTH];
} chat_t;
//----------------------------------------------------------------------------
typedef struct {
	CRITICAL_SECTION cs;
	u8 password[password_max_length];
	u8 passwordMix[password_max_length];
	u32 passwordLength;
	u32 passwordTimeOut;
	HANDLE thPassword;
	HANDLE eStopThreadPassword;
	SECRET_DATA sdata;
	KADEMLIA* kad;
	NETWORK* net;
	USERS* usr;
	CRITICAL_SECTION chat_cs;
	chat_t* chats;
	u32 chats_num;
	HANDLE thVoice;
	HANDLE eStop;
	u32(*trust_users)[NETWORK_ID_LENGTH];
	u32 trust_users_num;
	HWND hMainWnd;
} instance_t;
//----------------------------------------------------------------------------
static WAVEIN win;
static WAVEOUT wout;
static bool bWindows7;
static instance_t* inst; // must be static for access without message transfer
//----------------------------------------------------------------------------
KADEMLIA* getKad(void)
{
	return inst->kad;
}
//----------------------------------------------------------------------------
NETWORK* getNet(void)
{
	return inst->net;
}
//----------------------------------------------------------------------------
SECRET_DATA* getSecretData(void)
{
	return &inst->sdata;
}
//----------------------------------------------------------------------------
DWORD WINAPI voiceProc(instance_t* inst)
{
	s16 *buf_in, *buf_out;

	OSVERSIONINFO ver;
	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&ver);
	bWindows7 = ver.dwMajorVersion > 5;

	while(WAIT_TIMEOUT == WaitForSingleObject(inst->eStop,0))
	{
		if (WAIT_TIMEOUT != WaitForSingleObject(win.rxEvent,100) && (buf_in = (s16*)win.GetBuffer()))
		{
			buf_out = (s16*)wout.GetBuffer();
			if (buf_out) memset(buf_out,0,sizeof(s16)*BufferSize);
			//EnterCriticalSection(&inst->chat_cs);
			for (u32 i = 0; i < inst->chats_num; i++)
			{
				if (inst->chats[i].hWnd) SendMessageTimeout(inst->chats[i].hWnd,WM_CHANGE_VOICE,(WPARAM)buf_in,(LPARAM)buf_out,0,100,NULL);
			}
			//LeaveCriticalSection(&inst->chat_cs);
		}
	}
	return 0;
}
//----------------------------------------------------------------------------
void setVolume(WORD value)
{
	DWORD volume;
	if (value >= 100) volume = 0xFFFF;
	else volume = (DWORD)value*0xFFFF/100;
	volume |= volume<<16;
	waveOutSetVolume(wout.GetWaveOutHandle(),volume);
}
//----------------------------------------------------------------------------
void getVolume(WORD *value)
{
	DWORD volume;
	waveOutGetVolume(wout.GetWaveOutHandle(),&volume);
	*value = (DWORD)((volume&0xFFFF)*100/0xFFFF);
}

// Set the record input, volume and balance
static void MixerParametersXP(HWAVEIN hWaveIn, WORD* volume_in_percent, bool set)
{
 UINT i;
 HMIXER mix;
 MIXERLINE mixerDstLine;
 MIXERLINE mixerSrcLine;
 MIXERCONTROL mixerCtrl;
 MIXERLINECONTROLS mixerLineCtrl;
 MIXERCONTROLDETAILS mixerCtrlDetails;
 MIXERCONTROLDETAILS_UNSIGNED mixerCtrlDetailsUnsigned;

 // Open the mixer
 if (MMSYSERR_NOERROR != mixerOpen(&mix, (UINT)hWaveIn, 0, 0, MIXER_OBJECTF_HWAVEIN))
  {
  return;
  }
  
 // Get wavein
 mixerDstLine.cbStruct = sizeof(MIXERLINE);
 mixerDstLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

 if (MMSYSERR_NOERROR != mixerGetLineInfo((HMIXEROBJ)mix, &mixerDstLine, MIXER_GETLINEINFOF_COMPONENTTYPE))
  {
  mixerClose(mix);
  return;
  }

 // Get line input
 for (i = 0; i < mixerDstLine.cConnections; i++)
 {
  mixerSrcLine.cbStruct = sizeof(MIXERLINE);
  mixerSrcLine.dwDestination = mixerDstLine.dwDestination;
  mixerSrcLine.dwSource = i;

  mixerGetLineInfo((HMIXEROBJ)mix, &mixerSrcLine, MIXER_GETLINEINFOF_SOURCE);

  // if source is line in 
  if (mixerSrcLine.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE) break;
 }

 if (mixerSrcLine.dwComponentType != MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE)
  {
  mixerClose(mix);
  return;
  }

 // Get switch control
 mixerLineCtrl.cbStruct = sizeof(MIXERLINECONTROLS);
 mixerLineCtrl.dwLineID = mixerDstLine.dwLineID;
 mixerLineCtrl.dwControlType = MIXERCONTROL_CONTROLTYPE_MUX;
 mixerLineCtrl.cControls = 1;
 mixerLineCtrl.pamxctrl = &mixerCtrl;
 mixerLineCtrl.cbmxctrl = sizeof(MIXERCONTROL);

 if (MMSYSERR_NOERROR != mixerGetLineControls((HMIXEROBJ)mix, &mixerLineCtrl, MIXER_GETLINECONTROLSF_ONEBYTYPE))
  {
  mixerClose(mix);
  return;
  }

 // select line input
 if (mixerCtrl.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE) // multiple controle case
 {
  MIXERCONTROLDETAILS_UNSIGNED *mux_vals = new MIXERCONTROLDETAILS_UNSIGNED[mixerCtrl.cMultipleItems];

  for (i = 0; i < mixerCtrl.cMultipleItems; i++)
  {
   if (mixerSrcLine.dwSource == i) mux_vals[i].dwValue = 1; // set corresponding input to enable state
   else mux_vals[i].dwValue = 0;
  }

  mixerCtrlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
  mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
  mixerCtrlDetails.cChannels = 1;
  mixerCtrlDetails.cMultipleItems = mixerCtrl.cMultipleItems;
  mixerCtrlDetails.paDetails = mux_vals;
  mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);

  if (MMSYSERR_NOERROR != mixerSetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_SETCONTROLDETAILSF_VALUE))
   {
   mixerClose(mix);
   delete[] mux_vals;
   return;
   }

  delete[] mux_vals;
 }
 else // single controle case
 {
  mixerCtrlDetailsUnsigned.dwValue = mixerSrcLine.dwSource;

  mixerCtrlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
  mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
  mixerCtrlDetails.cMultipleItems = 0;
  mixerCtrlDetails.paDetails = &mixerCtrlDetailsUnsigned;
  mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
  mixerCtrlDetails.cChannels = 1;

  if (MMSYSERR_NOERROR != mixerSetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_SETCONTROLDETAILSF_VALUE))
   {
   mixerClose(mix);
   return;
   }
 }

 // Get the volume control
 mixerLineCtrl.cbStruct = sizeof(MIXERLINECONTROLS);
 mixerLineCtrl.dwLineID = mixerSrcLine.dwLineID;
 mixerLineCtrl.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
 mixerLineCtrl.cControls = 1;
 mixerLineCtrl.pamxctrl = &mixerCtrl;
 mixerLineCtrl.cbmxctrl = sizeof(MIXERCONTROL);

 if (MMSYSERR_NOERROR != mixerGetLineControls((HMIXEROBJ)mix, &mixerLineCtrl, MIXER_GETLINECONTROLSF_ONEBYTYPE))
  {
  mixerClose(mix);
  return;
  }

mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
mixerCtrlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
mixerCtrlDetails.cMultipleItems = 0;
mixerCtrlDetails.paDetails = &mixerCtrlDetailsUnsigned;
mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
mixerCtrlDetails.cChannels = 1;
 if (set)
 {
	 // Set volume
	 mixerCtrlDetailsUnsigned.dwValue = mixerCtrl.Bounds.dwMinimum + (mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum)*(*volume_in_percent)/100;

	 if (MMSYSERR_NOERROR != mixerSetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_SETCONTROLDETAILSF_VALUE))
	  {
	  mixerClose(mix);
	  return;
	  }
 }
 else
 {
	 // Get volume
	 if (MMSYSERR_NOERROR != mixerGetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_GETCONTROLDETAILSF_VALUE))
	  {
	  mixerClose(mix);
	  return;
	  }
	 *volume_in_percent = (WORD)((mixerCtrlDetailsUnsigned.dwValue - mixerCtrl.Bounds.dwMinimum)*100/(mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum));
 }

 // close the mixer
 mixerClose(mix);
}

// Set the record input, volume and balance
static void MixerParametersW7(HWAVEIN hWaveIn, WORD* volume_in_percent, bool set)
{
 HMIXER mix;
 MIXERLINE mixerLine;
 MIXERCONTROL mixerCtrl;
 MIXERLINECONTROLS mixerLineCtrl;
 MIXERCONTROLDETAILS mixerCtrlDetails;
 MIXERCONTROLDETAILS_UNSIGNED mixerCtrlDetailsUnsigned;

 // Open the mixer
 if (MMSYSERR_NOERROR != mixerOpen(&mix, (UINT)hWaveIn, 0, 0, MIXER_OBJECTF_HWAVEIN))
  {
  return;
  }
  
 // Get wavein
 mixerLine.cbStruct = sizeof(MIXERLINE);
 mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

 if (MMSYSERR_NOERROR != mixerGetLineInfo((HMIXEROBJ)mix, &mixerLine, MIXER_GETLINEINFOF_COMPONENTTYPE))
  {
  mixerClose(mix);
  return;
  }

 // Get the volume control
 mixerLineCtrl.cbStruct = sizeof(MIXERLINECONTROLS);
 mixerLineCtrl.dwLineID = mixerLine.dwLineID;
 mixerLineCtrl.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
 mixerLineCtrl.cControls = 1;
 mixerLineCtrl.pamxctrl = &mixerCtrl;
 mixerLineCtrl.cbmxctrl = sizeof(MIXERCONTROL);

 if (MMSYSERR_NOERROR != mixerGetLineControls((HMIXEROBJ)mix, &mixerLineCtrl, MIXER_GETLINECONTROLSF_ONEBYTYPE))
  {
  mixerClose(mix);
  return;
  }

mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
mixerCtrlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
mixerCtrlDetails.cMultipleItems = 0;
mixerCtrlDetails.paDetails = &mixerCtrlDetailsUnsigned;
mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
mixerCtrlDetails.cChannels = 1;
 if (set)
 {
	 // Set volume
	 mixerCtrlDetailsUnsigned.dwValue = mixerCtrl.Bounds.dwMinimum + (mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum)*(*volume_in_percent)/100;

	 if (MMSYSERR_NOERROR != mixerSetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_SETCONTROLDETAILSF_VALUE))
	  {
	  mixerClose(mix);
	  return;
	  }
 }
 else
 {
	 // Get volume
	 if (MMSYSERR_NOERROR != mixerGetControlDetails((HMIXEROBJ)mix, &mixerCtrlDetails, MIXER_GETCONTROLDETAILSF_VALUE))
	  {
	  mixerClose(mix);
	  return;
	  }
	 *volume_in_percent = (WORD)((mixerCtrlDetailsUnsigned.dwValue - mixerCtrl.Bounds.dwMinimum)*100/(mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum));
 }

 // close the mixer
 mixerClose(mix);
}
//----------------------------------------------------------------------------
void setMicrophone(WORD value)
{
  if (bWindows7)
  {
   MixerParametersW7(win.GetWaveInHandle(), &value,true); // set mixer's mux to line and volume (in %)
  }
  else
  {
   MixerParametersXP(win.GetWaveInHandle(), &value,true); // set mixer's mux to line and volume (in %)
  }
}
//----------------------------------------------------------------------------
void getMicrophone(WORD* value)
{
  if (bWindows7)
  {
   MixerParametersW7(win.GetWaveInHandle(), value,false); // set mixer's mux to line and volume (in %)
  }
  else
  {
   MixerParametersXP(win.GetWaveInHandle(), value,false); // set mixer's mux to line and volume (in %)
  }
}

//----------------------------------------------------------------------------
DWORD WINAPI PasswordThread(void* param)
{
	instance_t* inst = (instance_t*)param;
	while (WAIT_TIMEOUT == WaitForSingleObject(inst->eStopThreadPassword,60000))
	{
		EnterCriticalSection(&inst->cs);
		if (!inst->passwordTimeOut && inst->passwordLength) // reset password
		{
			ZeroMemory(inst->password,sizeof(inst->password));
			ZeroMemory(inst->passwordMix,sizeof(inst->password));
			inst->passwordLength = 0;
			LeaveCriticalSection(&inst->cs);
			LogMessage(module,TEXT("Password Cleared"));
		}
		else if (inst->passwordTimeOut)
		{
			u32 i;
			inst->passwordTimeOut--;
			for (i = 0; i < inst->passwordLength; i++)
			{
				inst->password[i] ^= inst->passwordMix[i];
			}
			cheap_random_number(inst->passwordMix, (inst->passwordLength+3)/4);
			for (i = 0; i < inst->passwordLength; i++)
			{
				inst->password[i] ^= inst->passwordMix[i];
			}
			LeaveCriticalSection(&inst->cs);
			LogMessage(module,TEXT("Password Mixed"));
		}
	}
	return 0;
}
//----------------------------------------------------------------------------
u32 getPassword(u8* password)
{
	u32 pass_len = 0;
	u32 i;
	if (!inst->sdata.data_is_initialized) return 0;
	// try to get remembered password
	EnterCriticalSection(&inst->cs);
	if (inst->passwordLength)
	{
		for (i = 0; i < inst->passwordLength; i++)
		{
			password[i] = inst->password[i]^inst->passwordMix[i];
		}
		pass_len = inst->passwordLength;
	}
	LeaveCriticalSection(&inst->cs);

	if (pass_len) 
	{
		LogMessage(module,TEXT("Get password: stored value"));
		return pass_len; // password exists
	}

	u32 cnt = 5;
	while(cnt--)
	{
		pass_len = PasswordRequest(inst->hMainWnd,password);
		if (!pass_len) return 0;
		if (check_password(&inst->sdata.encoded_secret_key,password,pass_len))
		{
			settings_t *set = (settings_t*)GetProp(inst->hMainWnd,TEXT("set"));
			if (set->passwordTime)
			{
				EnterCriticalSection(&inst->cs);
				inst->passwordTimeOut = set->passwordTime;
				inst->passwordLength = pass_len;
				cheap_random_number(inst->passwordMix, pass_len);
				for (i = 0; i < inst->passwordLength; i++)
				{
					inst->password[i] = password[i]^inst->passwordMix[i];
				}
				LeaveCriticalSection(&inst->cs);
			}
			LogMessage(module,TEXT("Get password: user is requested"));
			return pass_len;
		}
		else MessageBox(inst->hMainWnd,TEXT("Password doesn't match key\nTry again"),TEXT("Password"),MB_OK|MB_ICONEXCLAMATION);
	}
	MessageBox(inst->hMainWnd,TEXT("Too many attempts to enter the password\nPROGRAM WILL BE CLOSED"),TEXT("Error"),MB_OK|MB_ICONERROR);
	LogMessage(module,TEXT("USER ENTER WRONG PASSWORD MANY TIMES PROGRAM WILL BE CLOSED"));
	DestroyWindow(inst->hMainWnd);
	return 0;
}
//----------------------------------------------------------------------------
void parsePacket(NET_PACKET* pac)
{
	u32 i;
    switch(pac->buffer[0])
    {
    case NETWORK_PROT_KADEMLIA:
		inst->kad->packetReceived(&pac->buffer[1],pac->buf_len-sizeof(u32),pac->nd,inst->usr);
        break;
    case NETWORK_PROT_CONTROL:
		EnterCriticalSection(&inst->chat_cs);
		for (i = 0; i < inst->chats_num; i++)
		{
			if (inst->chats[i].hWnd && isIdIdentical(pac->nd->id,inst->chats[i].id)) break;
		}
		if (i >= inst->chats_num) // create new chat
		{
			LeaveCriticalSection(&inst->chat_cs);
			// have to create new window in appropriate thread!!!
			SendMessage(inst->hMainWnd,WM_CREATECHAT,0,(LPARAM)pac->nd->id);
			break;
			/*
			TCHAR user[user_info_max_length_bytes/sizeof(TCHAR)];
			for (i = 0; i < inst->chats_num; i++) // try to find empty space
			{
				if (!inst->chats[i].hWnd) break;
			}
			if (i >= inst->chats_num) // allocate space
			{
				inst->chats_num += 20;
				chat_t* ch = (chat_t*)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,inst->chats_num*sizeof(chat_t));
				if (ch && inst->chats)
				{
					CopyMemory(ch, inst->chats, (inst->chats_num-20)*sizeof(chat_t));
				}
				if (inst->chats) HeapFree(GetProcessHeap(),0,inst->chats);
				inst->chats = ch;
				i = inst->chats_num-20;
			}
			LeaveCriticalSection(&inst->chat_cs);
			memcpy(inst->chats[i].id,pac->nd->id,NETWORK_ID_LENGTH*sizeof(u32));
			inst->chats[i].hWnd = StartChat(inst->hMainWnd,pac->nd);
			inst->net->get_user_info(pac->nd,user);
			SetWindowText(inst->chats[i].hWnd,user);
			*/
		}
		else
			LeaveCriticalSection(&inst->chat_cs);
		SendMessage(inst->chats[i].hWnd,WM_PROCESSDATA,pac->buf_len-sizeof(u32),(LPARAM)&pac->buffer[1]);
        break;
	default:
	    // another protocols here
		break;
	}
}
//----------------------------------------------------------------------------
bool isTrustUser(u32* id)
{
	for (u32 i = 0; i < inst->trust_users_num; i++)
	{
		if (isIdIdentical(id,inst->trust_users[i])) return true;
	}
	return false;
}
//----------------------------------------------------------------------------
static void updateTable(HWND hWnd)
{
	like_t* ptr;
	TCHAR str[100];
	HWND hUsers = GetDlgItem(hWnd,IDC_USERS);
    int cnt = 0;
	int cur_sel = ListView_GetSelectionMark(hUsers);
	int num = ListView_GetItemCount(hUsers);
	u32 sel_id[NETWORK_ID_LENGTH];
	if (!ListView_GetSelectedCount(hUsers))
	{
		cur_sel = -1;
	}
	if (cur_sel >= 0)
	{
		ListView_GetItemText(hUsers,cur_sel,3,str,100);
		str2id(str,sel_id);
	}
	for(ptr = inst->usr->getLikes(); ptr; ptr = ptr->next)
	{
		LVITEM item = {0};

		inst->net->update_ip(&ptr->nd);
			
		item.mask = LVIF_TEXT|LVIF_IMAGE;
		item.pszText = ptr->name;
		item.iImage = isTrustUser(ptr->nd.id)?1:0;
		item.iItem = cnt++;
		if (num < cnt) item.iItem = ListView_InsertItem(hUsers,&item);
		else ListView_SetItem(hUsers,&item);
		item.iSubItem = 1;
		item.pszText = str;
		ip2str(ptr->nd.ip,str);
		item.iImage = inst->net->is_user_online(ptr->nd.id)?1:0;
		ListView_SetItem(hUsers,&item);
		item.mask = LVIF_TEXT;
		item.iSubItem = 2;
		_stprintf_s(str,100,TEXT("%d"),ptr->nd.port);
		ListView_SetItem(hUsers,&item);
		item.iSubItem = 3;
		id2str(ptr->nd.id,str);
		ListView_SetItem(hUsers,&item);

		if ((cur_sel >= 0) && isIdIdentical(sel_id,ptr->nd.id))
		{
			ListView_SetItemState(hUsers,item.iItem,LVIS_SELECTED,LVIS_SELECTED);
			ListView_SetSelectionMark(hUsers,item.iItem);
		}
	}
	// clear all other items if exist
	while(cnt < (num = ListView_GetItemCount(hUsers)))
	{
		ListView_DeleteItem(hUsers,num-1);
	}
	ListView_RedrawItems(hUsers,0,cnt);
}
//----------------------------------------------------------------------------
void GetFontForItem(HWND hWnd,int iItem)
{
	TCHAR str[100];
	u32 id[NETWORK_ID_LENGTH];
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),iItem,3,str,100);
	if (!str2id(str,id)) return;

}
//----------------------------------------------------------------------------
void AddTrustId(u32* id)
{
	u32(*trust_users)[NETWORK_ID_LENGTH];
	trust_users = (u32(*)[NETWORK_ID_LENGTH])HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,(inst->trust_users_num+1)*NETWORK_ID_LENGTH*sizeof(u32));
	CopyMemory(trust_users,inst->trust_users,inst->trust_users_num*NETWORK_ID_LENGTH*sizeof(u32));
	if (inst->trust_users) HeapFree(GetProcessHeap(),0,inst->trust_users);
	inst->trust_users = trust_users;
	CopyMemory(inst->trust_users[inst->trust_users_num],id,NETWORK_ID_LENGTH*sizeof(u32));
	inst->trust_users_num++;
}
//----------------------------------------------------------------------------
void RemoveTrustId(u32* id)
{
	for (u32 i = 0; i < inst->trust_users_num; i++)
	{
		if (isIdIdentical(inst->trust_users[i],id))
		{
			inst->trust_users_num--;
			MoveMemory(inst->trust_users[i],inst->trust_users[i+1],(inst->trust_users_num-i)*NETWORK_ID_LENGTH*sizeof(u32));
			break;
		}
	}
}
void appendLike(NODE* nd, pctstr_t name)
{
	inst->usr->append_like(nd,name);
}
void removeLike(u32* id)
{
	inst->usr->remove_like(id);
}
//----------------------------------------------------------------------------
static void userMenu(HWND hWnd,LPNMITEMACTIVATE ntf)
{
	TCHAR str[100];
	HMENU menu = CreatePopupMenu();
	MENUITEMINFO item;
	u32 id[NETWORK_ID_LENGTH];

	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),ntf->iItem,3,str,100);
	if (!str2id(str,id)) return;

	item.cbSize = sizeof(MENUITEMINFO);
	item.fMask = MIIM_TYPE|MIIM_ID;
	item.fType = MFT_STRING;
	if (isTrustUser(id))
	{
		item.wID = IDM_REMOVETRUST;
		item.dwTypeData = TEXT("Remove from trust list");
	}
	else
	{
		item.wID = IDM_ADDTRUST;
		item.dwTypeData = TEXT("Add to trust list");
	}
	InsertMenuItem(menu,0,TRUE,&item);
	item.wID = IDM_REMOVESAVED;
	item.dwTypeData = TEXT("Remove from saved list");
	InsertMenuItem(menu,1,TRUE,&item);
	ClientToScreen(GetDlgItem(hWnd,IDC_USERS),&ntf->ptAction);
	switch (TrackPopupMenu(menu,TPM_RETURNCMD,ntf->ptAction.x,ntf->ptAction.y,0,hWnd,NULL))
	{
	case IDM_ADDTRUST:
		AddTrustId(id);
		break;
	case IDM_REMOVETRUST:
		RemoveTrustId(id);
		break;
	case IDM_REMOVESAVED:
		inst->usr->remove_like(id);
		break;
	}
}
//----------------------------------------------------------------------------
static void selectUser(HWND hWnd, u32* id)
{
	TCHAR str[100];
	u32 item_id[NETWORK_ID_LENGTH];
	for (int i = 0; i < ListView_GetItemCount(GetDlgItem(hWnd,IDC_USERS)); i++)
	{
		ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),i,3,str,100);
		if (!str2id(str,item_id)) return;
		if (isIdIdentical(id,item_id))
		{
			ListView_SetItemState(GetDlgItem(hWnd,IDC_USERS),i,LVIS_SELECTED,LVIS_SELECTED);
			ListView_SetSelectionMark(GetDlgItem(hWnd,IDC_USERS),i);
			break;
		}
	}
}
//----------------------------------------------------------------------------
static INT_PTR CALLBACK FindDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		SetProp(hWnd,TEXT("str"),(HANDLE)lParam);
		Edit_LimitText(GetDlgItem(hWnd,IDC_FIND),user_info_max_length_bytes/sizeof(TCHAR));
		return TRUE;
    case WM_DESTROY:
		RemoveProp(hWnd,TEXT("str"));
		return 0;
    case WM_CLOSE:
		EndDialog(hWnd,IDCANCEL);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			GetDlgItemText(hWnd,IDC_FIND,(LPTSTR)GetProp(hWnd,TEXT("str")),user_info_max_length_bytes/sizeof(TCHAR));
			EndDialog(hWnd,IDOK);
			return TRUE;
		case IDCANCEL:
			EndDialog(hWnd,IDCANCEL);
			return TRUE;
		}
		return FALSE;
	}
	return FALSE;
}
//----------------------------------------------------------------------------
static void findUser(HWND hWnd)
{
	TCHAR str[user_info_max_length_bytes/sizeof(TCHAR)];
	u32 id[NETWORK_ID_LENGTH];
	NODE nd;
	if (IDOK == DialogBoxParam(GetModuleHandle(NULL),TEXT("find"),hWnd,FindDialog,(LPARAM)str))
	{
		LogMessage(TEXT("Find"),str);
		if (NETWORK_ID_LENGTH == _stscanf_s(str,TEXT("%08X %08X %08X %08X %08X %08X %08X %08X"),&id[0],&id[1],&id[2],&id[3],&id[4],&id[5],&id[6],&id[7]))
		{
			LogMessage(TEXT("Find"),TEXT("User ID typed"));
			if (inst->kad->findNode(id,&nd))
			{
				selectUser(hWnd,id);
				LogMessage(TEXT("Find"),TEXT("User found"));
			}
		}
		else
		{
			LogMessage(TEXT("Find"),TEXT("User name typed"));
			if (inst->net->findUserByName((u8*)str,_tcslen(str)*sizeof(TCHAR),id))
			{
				selectUser(hWnd,id);
				LogMessage(TEXT("Find"),TEXT("User found"));
			}
		}
	}
}
//----------------------------------------------------------------------------
void StartChat(u32* id)
{
	TCHAR UserName[200];
	u32 i;
	NODE nd;
	// try to find same window
	EnterCriticalSection(&inst->chat_cs);
	for (i = 0; i < inst->chats_num; i++)
	{
		if (inst->chats[i].hWnd && isIdIdentical(id,inst->chats[i].id))
		{
			SetForegroundWindow(inst->chats[i].hWnd);
			LeaveCriticalSection(&inst->chat_cs);
			return;
		}
	}
	// try to find empty space
	for (i = 0; i < inst->chats_num; i++)
	{
		if (!inst->chats[i].hWnd) break;
	}
	if (i >= inst->chats_num)
	{
		// allocate space
		inst->chats_num += 20;
		chat_t* ch = (chat_t*)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,inst->chats_num*sizeof(chat_t));
		if (ch && inst->chats)
		{
			CopyMemory(ch, inst->chats, (inst->chats_num-20)*sizeof(chat_t));
		}
		if (inst->chats) HeapFree(GetProcessHeap(),0,inst->chats);
		inst->chats = ch;
	}
	LeaveCriticalSection(&inst->chat_cs);
	if (inst->net->findNode(id,&nd) || inst->kad->findNode(id,&nd))
	{
		memcpy(inst->chats[i].id,nd.id,NETWORK_ID_LENGTH*sizeof(u32));
		inst->chats[i].hWnd = StartChat(inst->hMainWnd,&nd);
		inst->net->get_user_info(&nd,UserName);
		SetWindowText(inst->chats[i].hWnd,UserName);
	}
}
//----------------------------------------------------------------------------
static void ActivateChat(HWND hWnd, int iItem)
{
	TCHAR str[100];
	u32 id[NETWORK_ID_LENGTH];
	ListView_GetItemText(GetDlgItem(hWnd,IDC_USERS),iItem,3,str,100);
	if (str2id(str,id))
	{
		StartChat(id);
	}
}
//----------------------------------------------------------------------------
static void LoadTrustUsers(HWND hWnd, instance_t* inst, LPTSTR fileName)
{
	FILE* F;
	if (!_tcslen(fileName)) return;
	if (_tfopen_s(&F,fileName,TEXT("rb")))
	{
		MessageBox(hWnd,TEXT("Can't open file with list of trust users"), TEXT("Warning"), MB_OK|MB_ICONWARNING);
		inst->trust_users = NULL;
		inst->trust_users_num = 0;
		return;
	}
	fseek(F,0,SEEK_END);
	inst->trust_users_num = ftell(F)/(NETWORK_ID_LENGTH*sizeof(u32));
	inst->trust_users = (u32(*)[NETWORK_ID_LENGTH])HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,inst->trust_users_num*NETWORK_ID_LENGTH*sizeof(u32));
	rewind(F);
	fread(inst->trust_users,NETWORK_ID_LENGTH*sizeof(u32),inst->trust_users_num,F);
	fclose(F);
}
//----------------------------------------------------------------------------
static void SaveTrustUsers(HWND hWnd, instance_t* inst, LPTSTR fileName)
{
	FILE* F;
	if (!_tcslen(fileName)) return;
	if (_tfopen_s(&F,fileName,TEXT("wb")))
	{
		MessageBox(hWnd,TEXT("Can't open file for list of trust users writing"), TEXT("Warning"), MB_OK|MB_ICONWARNING);
		return;
	}
	fwrite(inst->trust_users,NETWORK_ID_LENGTH*sizeof(u32),inst->trust_users_num,F);
	fclose(F);
}
//----------------------------------------------------------------------------
static void DeleteChat(HWND hWnd, HWND chat)
{
	u32 i;
	EnterCriticalSection(&inst->chat_cs);
	for (i = 0; i < inst->chats_num; i++)
	{
		if (inst->chats[i].hWnd == chat)
		{
			inst->chats[i].hWnd = NULL;
			break;
		}
	}
	LeaveCriticalSection(&inst->chat_cs);
}
//----------------------------------------------------------------------------
static void ShowAbout(HWND hWnd)
{
const LPTSTR about = TEXT("ZAS Telephone Beta 0.2 ") BUILD_NUM TEXT(" ") DATE_NOW TEXT("\r\n\r\n")
					TEXT("To all Freedom loving people of the World. ")
					TEXT("We do our best to provide you private communication in the upcoming darkness of the Orwellian age. ")
					TEXT("The software binaries are free and always be free for any use. The source code is included only for review purpose. ")
					TEXT("You are not allowed to modify any part of this software or source code or use it for your products unless we give you explicit permission.\r\n\r\n")
					TEXT("The software is provided as is. There are no warranties or liabilities of any kind.\r\n\r\n")
					TEXT("Report bugs and suggestions to:\r\n\r\n")
					TEXT("ZAS Telephone Team\r\n")
					TEXT("zas@zas-comm.ru\r\n\r\n")
					
TEXT("Key Type:  DSA/ELG\r\n")
TEXT("Algorithm: 3DES\r\n")
TEXT("\r\n")
TEXT("-----BEGIN PGP PUBLIC KEY BLOCK-----\r\n")
TEXT("mQGiBFJTL48RBADSPlffkPzBLDDP9wKo9RcPy8pcDNjjVECBUyydDT+IwfrBSDt9\r\n")
TEXT("YzH6lPUS7je69VppMg1EP2I0fMuDmlMmhNg9kEJUKk2kH+IqUIBPwHx+/g59qeQW\r\n")
TEXT("uoOZoYsY2D7AQB+gBmDJhviEfhSbeRi/9vMQ6Kbbz38v03Da1krBjZ1NIQCg/3cg\r\n")
TEXT("wNpuGMqHtWp840p+ihHuhQsD/2Nj9gpQIdE2WaezVjvbpUx+Yip0B0ZaaFJWWJi5\r\n")
TEXT("MBedsgWUrrcPIM7/dN/ErtmsWQuVpZiVhYxyYPjz0M1E6NdatxPFlzJXByUYaxLJ\r\n")
TEXT("OZaQvvq935Mpz8mMzia40Bxfx1d8uCBOErJ+7sL/Ai0xf/H3qI1uXNLaaQSsD0AM\r\n")
TEXT("7PABA/98u9XGSfWay3w8QGSl8FhOqFQqJ4dFLkQ8/Pr66DtYoazyAQF854AEAekh\r\n")
TEXT("I71sCH1aDOlyIgSgk0UrQKeZLi5I+k4NceEm+IHsFPTJj79yz7HoRXDwELpNSa1U\r\n")
TEXT("L6uFk5TeBPgUOOBtDyrfxqfBcY5rxCwfxHtHqTUTdLgBfkin+bQWWkFTIDwgemFz\r\n")
TEXT("QHphcy1jb21tLnJ1PokASwQQEQIACwUCUlMvjwQLAgMBAAoJEJ4yzI4hvppFrNsA\r\n")
TEXT("oMc5ORYgguoI+yCsy8FAe5ub3+YFAKDwtuAKb97fmXwjNMYUCPeyTPnAObkEDQRS\r\n")
TEXT("UzCYEBAA42fNXw9sW6hJQyGx5+QVLTMyl1zu1l9vVSEVwncowqQUml5BVFuSk06X\r\n")
TEXT("OktlTsb1XTQU8O9uJ7ewVTJH945Zp4jjBB3KMLfb9Z7g1jWZrMpbyLWvgXrc9s5A\r\n")
TEXT("0/zxLfbMfXkBGgNLD2mlbeOb+Am5FNW9j9As/g8/e4eTJAm7PdgjLKMX2lZ5Opy0\r\n")
TEXT("FCY0JCguUowonX+eq9q0enUgsdNd0oaUlL4G18i1Z6rF+/iutOkmAjMZXqPAOjMq\r\n")
TEXT("GtWLSbWE6r8YpZpnttE+I86hyIwdoIv+n+tY+TPSNeYe09Ljt+1UugR4YrnWTjko\r\n")
TEXT("ydFywOFcSvFNR2fwVCn8T6IOdYoG0rceLJwUK7v1b94igZlDH6e3wAW4qH37/P/y\r\n")
TEXT("TKUPd+KCHjKhOyoB8e2kukf0QLyXhE6rrvNqOi9gf55WNeLWRzNBGFbOKOwkmLdw\r\n")
TEXT("O5lSCDIoX7Ld1lvRYArUprO9biPwDL4qWvvdfihVB4fed/XI95Znu4sYwGtqChq+\r\n")
TEXT("gxKSiEsBuyBR0KwewP7J/g56aym0vpgThy6237N7BPlU67aEzVjGZ2K9HyPQHZQb\r\n")
TEXT("+KZMp+L5uVNYfjoSvf76tTgD91q6s0c3lOIv9q29ekvwHMpTs2QVjt/gCdLB8lvh\r\n")
TEXT("YeD7HvrWooeuDIz8a5SXjf/vaIbknnO4PghA8tvM1vZozoyjVj8AAgIP/R6dYDWH\r\n")
TEXT("+gW63OiLuAvV95DqtQPPafMYWtbTXvEAxjxyB8wtYJeCFmkxR3qLglIaHpLDTlUX\r\n")
TEXT("kB7btNSzytJYHwImOQrGdaGkyf1tzg3OEm+J7pV0ezCCQ4uxabgu6mHuD06wx/1/\r\n")
TEXT("ZEYArgk1UTK/w9Z94ZhB1SghsdtPOA/pmwK8q5pAXmwB7WrCNA74tKDTo1o2tuh/\r\n")
TEXT("xZcHBg4Q1mW8wn4v+ZZA6guahEOZi2VALtc7MQLOuttXmr2GO3FqaNvLsIf6KjyX\r\n")
TEXT("P6I3UrBjze8sM4KfN/d4vv3K0FQdrikLPytsDzQ5IAbYzOD9DUOMhPdk3zLt6LOy\r\n")
TEXT("mTk6pDvlj7YXBlkCjOdCsLQSiwxRqRermk1309oXq9Kjvj3vRUHbbI+DJvnXsAmH\r\n")
TEXT("+h1/wY/aoTaeeWaQhL0LkbHoD4VY4YLHpCBJhO6HRVVTMqNNjdI6w6CNyNkf81XQ\r\n")
TEXT("yiKh7tjoi1ykxFhlKnpoV9+g7xv7UFvlvUKuW/HKJTSL2iaKxCzF5W6wlNoiDweQ\r\n")
TEXT("JsgqGixqCPMNHWuIdQGk1oz3cYAMFKeJbJQzKCNg57NjspukM4MzDCfpJpY7VcOc\r\n")
TEXT("LaksCLTmoJXrWj+trOhTTAptewhMpiTE69Wpo4JlYNfQedcJ3kN62GJ3DGqN57L4\r\n")
TEXT("hN7AtVVdKdaOt2KjnK+FY3VI5WmQg4hTHg6piQBGBBgRAgAGBQJSUzCYAAoJEJ4y\r\n")
TEXT("zI4hvppFf4oAn14uRam5vMCiM8fivdMVUKTvPz9WAKCsWKvjUiVCbJRaDNhXes0/\r\n")
TEXT("5QGdbA==\r\n")
TEXT("=RUjC\r\n")
TEXT("-----END PGP PUBLIC KEY BLOCK-----");
	HIMAGELIST IL = ImageList_Create(128,128,ILC_COLOR32,1,1);
	ImageList_AddIcon(IL,LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_ABOUT)));
	SetProp(hWnd,TEXT("image"),IL);

	SetFocus(GetDlgItem(hWnd,IDOK));

	SetDlgItemText(hWnd,IDC_ABOUT_TEXT,about);
}
//----------------------------------------------------------------------------
static INT_PTR CALLBACK AboutDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		ShowAbout(hWnd);
		return TRUE;
    case WM_DESTROY:
		ImageList_Destroy((HIMAGELIST)RemoveProp(hWnd,TEXT("image")));
		return 0;
    case WM_CLOSE:
		EndDialog(hWnd,0);
		return TRUE;

    case WM_PAINT:
	{
		PAINTSTRUCT ps;
		HDC hdc = BeginPaint(hWnd, &ps);
		ImageList_Draw((HIMAGELIST)GetProp(hWnd,TEXT("image")),0,hdc,2,40,ILD_NORMAL);
        EndPaint(hWnd, &ps);
		return FALSE;
	}
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			EndDialog(hWnd,0);
			return TRUE;
		}
		return FALSE;
	}
	return FALSE;
}
//----------------------------------------------------------------------------
static void MainWndSizing(HWND hWnd, WPARAM wParam, LPRECT newWndRect)
{
	RECT oldWndRect;
	RECT oldClientRect;
	const LONG lMinWidth = 400;
	const LONG lMinHeight = 300;
	//
	GetWindowRect(hWnd, &oldWndRect);
	GetClientRect(hWnd, &oldClientRect);
	//
	oldClientRect.right -= oldWndRect.right - oldWndRect.left - (newWndRect->right - newWndRect->left);		// calculate new client width
	oldClientRect.bottom -= oldWndRect.bottom - oldWndRect.top - (newWndRect->bottom - newWndRect->top);	// calculate new client height
	// adjust width
	if (oldClientRect.right < lMinWidth){
		switch (wParam)
		{
		case WMSZ_BOTTOMLEFT:
		case WMSZ_LEFT:
		case WMSZ_TOPLEFT:
			newWndRect->left -= lMinWidth - oldClientRect.right;
			break;
			//
		default:
			newWndRect->right += lMinWidth - oldClientRect.right;
		}
	}
	// adjust height
	if (oldClientRect.bottom < lMinHeight){
		switch (wParam)
		{
		case WMSZ_TOPLEFT:
		case WMSZ_TOP:
		case WMSZ_TOPRIGHT:
			newWndRect->top -= lMinHeight - oldClientRect.bottom;
			break;
			//
		default:
			newWndRect->bottom += lMinHeight - oldClientRect.bottom;
		}
	}
}
//----------------------------------------------------------------------------
static void MainWndSize(HWND hWnd)
{
	RECT rc;
	GetClientRect(hWnd,&rc);

	MoveWindow(GetDlgItem(hWnd,IDC_USERS),0,0,rc.right,rc.bottom,TRUE);
}
//----------------------------------------------------------------------------
static void MainWndInit(HWND hWnd)
{
	LVITEM item = {0};
	LVCOLUMN col;
	HWND hSplash = CreateDialog(GetModuleHandle(NULL),TEXT("About"),hWnd,AboutDialog);
	HWND hUsers = CreateWindowEx(WS_EX_CLIENTEDGE,WC_LISTVIEW,TEXT(""),WS_CHILD|WS_VISIBLE|LVS_REPORT|LVS_SINGLESEL|LVS_SORTASCENDING,0,0,10,20,hWnd,(HMENU)IDC_USERS,GetModuleHandle(NULL),NULL);
	ListView_SetExtendedListViewStyle(hUsers,LVS_EX_FULLROWSELECT|LVS_EX_AUTOSIZECOLUMNS|LVS_EX_GRIDLINES|LVS_EX_SUBITEMIMAGES);
	// create columns
	col.mask = LVCF_WIDTH|LVCF_TEXT;
	col.pszText = TEXT("Name");
	col.cx = 100;
	ListView_InsertColumn(hUsers,0,&col);
	col.pszText = TEXT("IP");
	ListView_InsertColumn(hUsers,1,&col);
	col.pszText = TEXT("Port");
	ListView_InsertColumn(hUsers,2,&col);
	col.pszText = TEXT("ID");
	col.cx = 500;
	ListView_InsertColumn(hUsers,3,&col);
	// add image list
    HIMAGELIST hLarge = ImageList_Create(GetSystemMetrics(SM_CXICON),GetSystemMetrics(SM_CYICON),ILC_COLOR32,2,1);
    HIMAGELIST hSmall = ImageList_Create(GetSystemMetrics(SM_CXSMICON),GetSystemMetrics(SM_CYSMICON),ILC_COLOR32, 2, 1);
    HICON hiconItem = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_GREY_USER));
    ImageList_AddIcon(hLarge, hiconItem);
    ImageList_AddIcon(hSmall, hiconItem);
    DestroyIcon(hiconItem);
    hiconItem = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_GREEN_USER));
    ImageList_AddIcon(hLarge, hiconItem);
    ImageList_AddIcon(hSmall, hiconItem);
    DestroyIcon(hiconItem);
	ListView_SetImageList(hUsers,hLarge,LVSIL_NORMAL);
	ListView_SetImageList(hUsers,hSmall,LVSIL_SMALL);
	// settings
	settings_t *set = (settings_t*)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(settings_t));
	SetProp(hWnd,TEXT("set"),set);
	LoadSettings(hWnd,set);
	CreateDialogParam(GetModuleHandle(NULL),TEXT("Log"),hWnd,LogProc,(LPARAM)set);
	
	//instance
	inst = (instance_t*)VirtualAlloc(NULL,sizeof(instance_t),MEM_RESERVE|MEM_COMMIT,PAGE_READWRITE);
	if (!VirtualLock(inst,sizeof(instance_t))) // For security reason this memory must not be written to the pagefile
	{
		MessageBox(hWnd,TEXT("Can't lock secure data memory"),TEXT("Warning"),MB_OK|MB_ICONWARNING);
	}
	inst->hMainWnd = hWnd;
	InitializeCriticalSection(&inst->cs);
	InitializeCriticalSection(&inst->chat_cs);
	LoadKey(hWnd,set->keyFileName,&inst->sdata);
	NODE me;
	CopyMemory(me.id,inst->sdata.certificate.fingerprint,NETWORK_ID_LENGTH*sizeof(u32));
	me.ip = MAKEIPADDRESS(127,0,0,1);
	me.port = set->localPort;
	inst->kad = new KADEMLIA(&me,hWnd);
	inst->net = new NETWORK(&me,hWnd);
	inst->usr = new USERS();
	SetTimer(hWnd,1,1000,NULL);
	inst->eStopThreadPassword = CreateEvent(NULL,TRUE,FALSE,NULL);
	inst->thPassword = CreateThread(NULL,0,PasswordThread,inst,0,NULL);
	inst->eStop = CreateEvent(NULL,TRUE,FALSE,NULL);
	inst->thVoice = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)voiceProc,inst,0,NULL);
	LoadTrustUsers(hWnd,inst,set->fgpFileName);
	if (set->enableUPnP)
	{
		if (!setupUPnP(set->localPort,set->remotePort,0))
		{
			MessageBox(hWnd,TEXT("Can't setup UPnP"),TEXT("Warning"),MB_OK|MB_ICONWARNING);
		}
	}
	inst->usr->load_nodes(inst->kad,set->userFileName);
	inst->usr->load_likes(set->likesFileName);
	CreateListsDialog(hWnd);
	DestroyWindow(hSplash);
	ShowWindow(g_logWnd,SW_NORMAL);
	ShowWindow(g_NetList,SW_NORMAL);
	ShowWindow(g_KadList,SW_NORMAL);
}
//----------------------------------------------------------------------------
static void MainWndDestroy(HWND hWnd)
{
	settings_t *set = (settings_t*)RemoveProp(hWnd,TEXT("set"));
	DestroyWindow(g_NetList);
	DestroyWindow(g_KadList);
	KillTimer(hWnd,1);
	SetEvent(inst->eStopThreadPassword);
	SetEvent(inst->eStop);
	WaitForSingleObject(inst->thPassword,1000);
	WaitForSingleObject(inst->thVoice,1000);
	CloseHandle(inst->eStopThreadPassword);
	CloseHandle(inst->eStop);
	CloseHandle(inst->thPassword);
	CloseHandle(inst->thVoice);
	inst->usr->save_nodes(set->userFileName);
	inst->usr->save_likes(set->likesFileName);
	if (inst->chats)
	{
		u32 i;
		for (i = 0; i < inst->chats_num; i++)
		{
			if (inst->chats[i].hWnd) DestroyWindow(inst->chats[i].hWnd);
		}
		HeapFree(GetProcessHeap(),0,inst->chats);
	}
	delete inst->kad;
	delete inst->net;
	delete inst->usr;
	DeleteCriticalSection(&inst->cs);
	DeleteCriticalSection(&inst->chat_cs);
	SaveSettings(hWnd,set);
	SaveTrustUsers(hWnd,inst,set->fgpFileName);
	if (inst->trust_users) HeapFree(GetProcessHeap(),0,inst->trust_users);
	ZeroMemory(inst->password,password_max_length); // clear password
	HeapFree(GetProcessHeap(),0,set);
	VirtualUnlock(inst,sizeof(instance_t));
	VirtualFree(inst,sizeof(instance_t),MEM_RELEASE);
	DestroyWindow(g_logWnd);
}
//----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_CREATE:
		MainWndInit(hWnd);
		return 0;
		//
    case WM_DESTROY:
		MainWndDestroy(hWnd);
		PostQuitMessage(0);
		return 0;
		//
	case WM_SIZE:
		MainWndSize(hWnd);
		return 0;
		//
	case WM_SIZING:
		MainWndSizing(hWnd, wParam, (LPRECT)lParam);
		return TRUE;
		//
	case WM_TIMER:
		switch (wParam)
		{
		case 1:
			updateTable(hWnd);
			return 0;
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDM_EXIT:
			DestroyWindow(hWnd);
			return 0;
		case IDM_ABOUT:
			DialogBox(GetModuleHandle(NULL),TEXT("About"),hWnd,AboutDialog);
			return 0;
		case IDM_HELP:
			return 0;
		case IDM_SETTINGS:
			SettingDialog(hWnd,(settings_t*)GetProp(hWnd,TEXT("set")),inst->usr,inst->kad);
			return 0;
		case IDM_GENERATE:
			GenerateKey(hWnd,((settings_t*)GetProp(hWnd,TEXT("set")))->keyFileName);
			return 0;
		case IDM_BOOTSTRAP:
			if (inst->net)
			{
				inst->net->reset_sessions();
			}
			if (inst->kad)
			{
				inst->kad->start();
				LogMessage(module,TEXT("Bootstrap started"));
			}
			return 0;
		case IDM_SHOW_LOG:
			ShowWindow(g_logWnd,SW_SHOWNORMAL);
			SetForegroundWindow(g_logWnd);
			return 0;
		case IDM_SHOW_NET:
			ShowWindow(g_NetList,SW_SHOWNORMAL);
			SetForegroundWindow(g_NetList);
			return 0;
		case IDM_SHOW_KAD:
			ShowWindow(g_KadList,SW_SHOWNORMAL);
			SetForegroundWindow(g_KadList);
			return 0;
		case IDM_FIND:
			findUser(hWnd);
			return 0;
		}
		return TRUE;
	case WM_NOTIFY:
	{
		LPNMITEMACTIVATE ntf = (LPNMITEMACTIVATE)lParam;
		if (ntf->hdr.idFrom == IDC_USERS)
		{
			switch (ntf->hdr.code)
			{
			case NM_DBLCLK:
				ActivateChat(hWnd,ntf->iItem);
				break;
			case NM_RCLICK:
				userMenu(hWnd,ntf);
				break;
			}
		}
		return FALSE;
	}
	case WM_SHOWTEXT:
		return 0;
	case WM_CREATECHAT:
		StartChat((u32*)lParam);
		return 0;
	case WM_DELETECHAT:
		DeleteChat(hWnd,(HWND)lParam);
		return 0;
	case WM_USER_FOUND:
	{
		NODE nd;
		inst->kad->findNode((u32*)lParam,&nd);
		selectUser(hWnd,(u32*)lParam);
		delete[] (u32*)lParam;
	}
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG AMsg;
	BOOL ret;
	WNDCLASSEX wc;
    INITCOMMONCONTROLSEX icex;
	HANDLE hMutex;

	// check that another copy of ZAS is running
	hMutex = CreateMutex(NULL, TRUE, TEXT("ZAS telephone"));
	if (!hMutex)
	{
		MessageBox(NULL,TEXT("Can't create mutex"),TEXT("Fatal Error"),MB_OK|MB_ICONERROR);
		return 0;
	}
	if (GetLastError() == ERROR_ALREADY_EXISTS)
	{
		MessageBox(NULL,TEXT("Another copy of ZAS telephone is running. This applicaton will be closed"),TEXT("Warning"),MB_OK|MB_ICONWARNING);
		return 0;
	}

	// common controls
	icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
   	icex.dwICC   = ICC_LISTVIEW_CLASSES|ICC_TAB_CLASSES|ICC_BAR_CLASSES|ICC_PROGRESS_CLASS|ICC_INTERNET_CLASSES;
   	InitCommonControlsEx(&icex);
	
	// window class
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hCursor = LoadCursor(NULL,IDC_ARROW);
	wc.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_MAIN));
	wc.hbrBackground = (HBRUSH)(COLOR_GRAYTEXT + 1);
	wc.hInstance = hInst;
	wc.lpszMenuName = MAKEINTRESOURCE(ID_MENU);
	wc.lpszClassName = TEXT("MainWindowClass");
	wc.hIconSm = LoadIcon(hInst, MAKEINTRESOURCE(IDI_MAIN));

	if (!RegisterClassEx(&wc)) 
		return 0;

	HWND hwndMain = CreateWindow(TEXT("MainWindowClass"),TEXT("ZAS Telephone"),WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,0,0,800,600,NULL,NULL,hInst,NULL);
	if (!hwndMain) return 0;

	ShowWindow(hwndMain,nCmdShow);

	HACCEL hAccel = LoadAccelerators(hInst, MAKEINTRESOURCE(ID_ACCEL));

	while ((ret = GetMessage(&AMsg, NULL, 0, 0)) != 0) {
		if (ret == -1)
			continue;
        if (!TranslateAccelerator(hwndMain, hAccel, &AMsg)) {
			BOOL trans = TRUE;
			u32 i;
			// check all modeless dialogs
			EnterCriticalSection(&inst->chat_cs);
			for (i = 0; (i < inst->chats_num) && trans; i++)
				trans = trans && !(inst->chats[i].hWnd && IsWindow(inst->chats[i].hWnd) && IsDialogMessage(inst->chats[i].hWnd,&AMsg));
			LeaveCriticalSection(&inst->chat_cs);
			if (trans && !IsDialogMessage(g_logWnd,&AMsg) && !IsDialogMessage(g_NetList,&AMsg) && !IsDialogMessage(g_KadList,&AMsg))
			{
				TranslateMessage(&AMsg);
				DispatchMessage(&AMsg);
			}
		}
	}

	CloseHandle(hMutex);

	return 0;
}
