#include <winsock2.h>
#include <Mswsock.h>
#include "network.h"
#include "password.h"
#include "resource.h"
#include "log.h"
#include "hash_func.h"
#include "random.h"
#include "diffie_helman.h"
#include "main.h"
#include <tchar.h>
#include <CommCtrl.h>
#include <Windows.h>
//-----------------------------------------
static LPTSTR module = TEXT("Network");
//-----------------------------------------
CONNECTION::CONNECTION(const NODE* node)
{
	CopyMemory(&nd,node,sizeof(NODE));
	flags = 0;
	next_connection = NULL;
	est = NULL;
	pack_num_est = 0;
	pack_num_sec = 0;
	pack_num_drop = 0;
	pack_num_sent = 0;
	cp_strcchcpy((tchar_t*)userInfo,sizeof(userInfo)/sizeof(tchar_t),_T("<unknown>"));
	userInfoSize = cp_strlen((tchar_t*)userInfo)*sizeof(tchar_t);
}
//-----------------------------------------
CONNECTION::~CONNECTION()
{
	if (est)
	{
		est->tx_session.reset();
		delete est;
	}
}
//-----------------------------------------------------------------------
DWORD WINAPI receiverProc(NETWORK* n);
//-----------------------------------------------------------------------
NETWORK::NETWORK(NODE* me, HWND mWnd)
{
	WSADATA wsaData;
	WSAStartup(MAKEWORD(2, 2), &wsaData);
	initialize_seed();
	InitializeCriticalSection(&cs);
	con = NULL;

	CopyMemory(my_id,me->id,NETWORK_ID_LENGTH*sizeof(u32));

    mainWnd = mWnd;
	recThread = NULL;

	// open new socket
	soc = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	if (soc == INVALID_SOCKET)
	{
		MessageBox(mWnd,TEXT("Can't create socket"),TEXT("Error"),MB_OK|MB_ICONERROR);
		return;
	}
	ZeroMemory(&my_addr,sizeof(my_addr));
	my_addr.sin_family = AF_INET;
	my_addr.sin_addr.s_addr = INADDR_ANY;
	my_addr.sin_port = htons(me->port);
	if (SOCKET_ERROR == bind(soc,(sockaddr*)&my_addr,sizeof(my_addr)))
	{
		MessageBox(mWnd,TEXT("Can't bind socket to local port"),TEXT("Error"),MB_OK|MB_ICONERROR);
		return;
	}
    // set receiver timeout in ms
	DWORD timeout = 100;
	setsockopt(soc,SOL_SOCKET,SO_RCVTIMEO,(char*)&timeout,sizeof(timeout));
	// disable  new behavior using
	// IOCTL: SIO_UDP_CONNRESET
	DWORD dwBytesReturned = 0;
	BOOL bNewBehavior = FALSE;
	WSAIoctl(soc, SIO_UDP_CONNRESET,&bNewBehavior, sizeof(bNewBehavior),NULL,0,&dwBytesReturned,NULL,NULL);
	// Initialize sending packet
    u8 passw[password_max_length];
	u32 pas_len = getPassword(passw);
	if (pas_len)
	{
		sending_size = cmn_tx_session.init_session((u8*)sending_buff, getSecretData(), passw, pas_len);
		// create receiver thread and stop event
		eStopThreads = CreateEvent(NULL,FALSE,FALSE,NULL);
		recThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)receiverProc,this,0,NULL);
	}
	else
	{
		recThread = NULL;
	}
}
//-----------------------------------------------------------------------
NETWORK::~NETWORK()
{
	if (recThread)
	{
		SetEvent(eStopThreads);
		WaitForSingleObject(recThread,1000);
		CloseHandle(recThread);
	}
	closesocket(soc);
	WSACleanup();
    DeleteCriticalSection(&cs);
	cmn_tx_session.reset();
	if (con) // clear all connections
	{
		while(con)
		{
			CONNECTION* cn = con->next_connection;
			delete con;
			con = cn;
		}
	}
	initialize_seed();
}
//-----------------------------------------------------------------------
CONNECTION* NETWORK::find_connection(const NODE* nd, bool establish)
{
	CONNECTION *s, *prev_s;
	if (!con) // there is no any session
	{
		if (establish)
		{
			con = new CONNECTION(nd);
			return con;
		}
		else
		{
			return NULL;
		}
	}
	for (s = con; s; s = s->next_connection)
	{
		if (isIdIdentical(s->nd.id,nd->id))
		{
			s->nd.ip = nd->ip;
			s->nd.port = nd->port;
			return s;
		}
		prev_s = s;
	}
	if (!establish) return NULL;
	prev_s->next_connection = new CONNECTION(nd);
	return prev_s->next_connection;
}
//-----------------------------------------------------------------------
void NETWORK::send_big_buffer(u32 ip, u16 port, u32* buf, u32 size, u32 command, u32* pack_cnt)
{
	u32 tx_buffer[NETWORK_SIZE_UDP/sizeof(u32)];
	sockaddr_in addr = {0};
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(ip);
	addr.sin_port = htons(port);
	
	CopyMemory(tx_buffer,my_id,NETWORK_ID_LENGTH*sizeof(u32));

	u32 total_pack = (size+NETWORK_SIZE_SECURED-1)/NETWORK_SIZE_SECURED; // number of packets for sending
	for (u32 p = 0; p < total_pack; p++)
	{
		u32 chunk_size = (p==(total_pack-1)) ? (size-(total_pack-1)*NETWORK_SIZE_SECURED) : NETWORK_SIZE_SECURED; // size of data chunk
		tx_buffer[NETWORK_ID_LENGTH] = command|(total_pack<<8)|(p<<16);
		CopyMemory(&tx_buffer[NETWORK_ID_LENGTH+1],&buf[p*NETWORK_SIZE_SECURED/sizeof(u32)],chunk_size);
		sendto(soc,(char*)tx_buffer,chunk_size+(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
	}
	*pack_cnt += total_pack;
}
//-----------------------------------------------------------------------
bool NETWORK::collect_big_buffer(u32* cur_buf, u32 size, ESTABLISHMENT* est)
{
	u32 total_pack = (cur_buf[0]>>8)&0xFF;
	u32 pack_num = (cur_buf[0]>>16)&0xFF;
	if ((pack_num >= total_pack) || (total_pack*NETWORK_SIZE_SECURED > NETWORK_SIZE_DH_MAX) ||
		((pack_num != (total_pack-1)) && (size != (NETWORK_SIZE_SECURED+4))) || 
		((pack_num == (total_pack-1)) && (size >= (NETWORK_SIZE_SECURED+4)))) // error
	{
		est->rec_flags = 0;
		return false;
	}
	CopyMemory(&est->buff[pack_num*NETWORK_SIZE_SECURED/sizeof(u32)],&cur_buf[1],size-4);
	est->rec_flags |= 1<<pack_num;
	if (pack_num == (total_pack-1)) // last packet
	{
		est->rec_size = (total_pack-1)*NETWORK_SIZE_SECURED + size-4;
	}
	for (u32 i = 0; i < total_pack; i++)
	{
		if (!(est->rec_flags & (1<<i))) return false;
	}
	return true;
}
//-----------------------------------------------------------------------
u32 NETWORK::send_packet(PACKET_FOR_SEND* pack)
{
	u32 size;
	u32 tx_buffer_plain[2048];
	u32 tx_buffer_coded[2048];
	CONNECTION* s;
	sockaddr_in addr = {0};
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(pack->nd->ip);
	addr.sin_port = htons(pack->nd->port);

	CopyMemory(tx_buffer_coded,my_id,NETWORK_ID_LENGTH*sizeof(u32));

    if (!isIdNonZero(pack->nd->id)) // for zero ID just send ping
    {
        tx_buffer_coded[NETWORK_ID_LENGTH] = SESSION_PING_REQ;
        sendto(soc,(char*)tx_buffer_coded,(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
        return 0;
    }

	EnterCriticalSection(&cs);
	s = find_connection(pack->nd,true);

	if (!s) // not enough memory
	{
		LeaveCriticalSection(&cs);
		return 0;
	}
	
	if(s->flags&SESSION_LAUNCHED)
	{
		tx_buffer_plain[0] = pack->protocol;
		CopyMemory(&tx_buffer_plain[1], pack->buffer, pack->buf_len);
		tx_buffer_coded[NETWORK_ID_LENGTH] = SESSION_ENCODED;
		size = s->ses.encrypt((u8*)&tx_buffer_coded[NETWORK_ID_LENGTH+1],(u8*)tx_buffer_plain,pack->buf_len+sizeof(u32));
        size = (u32)sendto(soc,(char*)tx_buffer_coded,size+(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
		s->pack_num_sent++;
		s->flags++;
		if ((s->flags&SESSION_REQ_COUNT_MASK) >= SESSION_REQ_COUNT_MAX) // so long time no any answer. Reset session
		{
			s->flags = 0;
			s->ses.reset();
		}
		LeaveCriticalSection(&cs);
	}
	else
	{
		if(!(s->flags&SESSION_PINGED) || !(s->flags&SESSION_RESPONDED)) // if node isn't pinged or didn't respond, send ping request command
		{
			tx_buffer_coded[NETWORK_ID_LENGTH] = SESSION_PING_REQ;
			s->flags |= SESSION_PINGED;
			sendto(soc,(char*)tx_buffer_coded,(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
			s->pack_num_sent++;
			LeaveCriticalSection(&cs);
		}
		else if(s->est) // pinged and responded, but didn't send answer yet. Resend request again
		{
            send_big_buffer(pack->nd->ip,pack->nd->port,sending_buff,sending_size,SESSION_REQUEST,&s->pack_num_sent);
			LeaveCriticalSection(&cs);
			TCHAR str[400];
			_stprintf_s(str,200,TEXT("Again send Session request to %d.%d.%d.%d:%d"),FIRST_IPADDRESS(pack->nd->ip),SECOND_IPADDRESS(pack->nd->ip),THIRD_IPADDRESS(pack->nd->ip),FOURTH_IPADDRESS(pack->nd->ip),pack->nd->port);
			LogMessage(module,str);
		}
		else // wrong condition. Pinged, responded but didn't create establishment
		{
			s->flags = 0;
			s->ses.reset();
			LeaveCriticalSection(&cs);
		}
		size = 0; // nothing was sent, not established connection
	}
	return size;
}
//-----------------------------------------------------------------------
s8 name_compare(const u8* buf_ref,const u8* buf_test,u32 max_size)
{
	while (max_size--)
	{
		s8 diff = *buf_ref++ - *buf_test++;
		if (diff) return diff;
	}
	return 0;
}
//-----------------------------------------------------------------------
DWORD WINAPI receiverProc(NETWORK* n)
{
	int pack_len;
	sockaddr_in addr = {0};
	CONNECTION* s;
	u32 size;
	RX_SECURE_SESSION rx_session;
	u32 session_id[fingerprint_length_x32];
	u32 buffer_plain[2048];
	u32 buffer_coded[2048];
	int fault_cnt = 10;

    while (WAIT_TIMEOUT == WaitForSingleObject(n->eStopThreads,0))
	{
		int addr_size = sizeof(addr);
		memset(&addr,0,sizeof(addr));
		addr.sin_family = AF_INET;
		pack_len = recvfrom(n->soc,(char*)buffer_coded,sizeof(buffer_coded),0,(sockaddr*)&addr,&addr_size);
		KADEMLIA* kad = getKad();
		if (pack_len == SOCKET_ERROR) // error parsing
        {
			int err;
			if ((err = WSAGetLastError()) == WSAETIMEDOUT) continue;
			if (!--fault_cnt) 
			{
				MessageBox(n->mainWnd,TEXT("To many attempts to open socket\nProgram will be closed"),TEXT("Error"),MB_OK|MB_ICONERROR);
				PostMessage(n->mainWnd,WM_CLOSE,0,0);
				return 0;
			}
			//MessageBox(n->mainWnd,TEXT("Socket error occures\nReopen socket"),TEXT("Error"),MB_OK|MB_ICONERROR);
			closesocket(n->soc);
			// open new socket
			n->soc = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
			if (n->soc == INVALID_SOCKET)
			{
				MessageBox(n->mainWnd,TEXT("Can't create socket"),TEXT("Error"),MB_OK|MB_ICONERROR);
				return 0;
			}
			if (SOCKET_ERROR == bind(n->soc,(sockaddr*)&n->my_addr,sizeof(n->my_addr)))
			{
				MessageBox(n->mainWnd,TEXT("Can't bind socket to local port"),TEXT("Error"),MB_OK|MB_ICONERROR);
				return 0;
			}
			// disable  new behavior using
			// IOCTL: SIO_UDP_CONNRESET
			DWORD dwBytesReturned = 0;
			BOOL bNewBehavior = FALSE;
			WSAIoctl(n->soc, SIO_UDP_CONNRESET,&bNewBehavior, sizeof(bNewBehavior),NULL,0,&dwBytesReturned,NULL,NULL);
			// set receiver timeout in ms
			DWORD timeout = 100;
			setsockopt(n->soc,SOL_SOCKET,SO_RCVTIMEO,(char*)&timeout,sizeof(timeout));
			continue;
        }
        if (pack_len >= (int)(NETWORK_ID_LENGTH+1)*sizeof(u32))
        {
            NODE nd;
			nd.ip = ntohl(addr.sin_addr.s_addr);
			nd.port = ntohs(addr.sin_port);
            CopyMemory(nd.id,buffer_coded,NETWORK_ID_LENGTH*sizeof(u32));
            EnterCriticalSection(&n->cs);
            s = n->find_connection(&nd,true);
			u8 pack_type = buffer_coded[NETWORK_ID_LENGTH]&0xFF;
            switch(pack_type)
            {
            case SESSION_REQUEST:
				s->pack_num_est++;
                if (!s->est)
                {
                    s->est = new ESTABLISHMENT;
                    s->est->rec_flags = 0;
                    s->est->rec_size = 0;
                    s->est->last_size = 0;
                    s->est->sent_size = 0;
                    s->est->tx_session = n->cmn_tx_session;
                }
                if (n->collect_big_buffer(&buffer_coded[NETWORK_ID_LENGTH],pack_len-NETWORK_ID_LENGTH*sizeof(u32),s->est))
                {
                    u8 passw[password_max_length];
					bool same_request = false;
					if (s->est->last_size && s->est->sent_size && (s->est->last_size == s->est->rec_size))
					{
						same_request = true;
						for (u32 i = 0; i < s->est->rec_size/4; i++)
							if (s->est->buff[i] != s->est->last_buf[i])
							{
								same_request = false;
								break;
							}
					}
					if (same_request)
					{
						n->send_big_buffer(nd.ip,nd.port,s->est->sent_buf,s->est->sent_size,SESSION_RESPONSE,&s->pack_num_sent);
						s->est->rec_size = 0;
						s->est->rec_flags = 0;
		                LeaveCriticalSection(&n->cs);
						TCHAR str[400];
						_stprintf_s(str,200,TEXT("Same Session request from %d.%d.%d.%d:%d"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port);
						LogMessage(module,str);
						break;
					}
					else
					{
						LeaveCriticalSection(&n->cs);
						u32 pas_len = getPassword(passw);
						EnterCriticalSection(&n->cs);
						s->est->sent_size = rx_session.response_to_init_session((u8*)s->est->sent_buf,(u8*)s->est->buff, getSecretData(), passw, pas_len);
						if ((s->est->sent_size > 0) && rx_session.session_is_active(session_id) && isIdIdentical(s->nd.id,session_id))
						{
							n->send_big_buffer(nd.ip,nd.port,s->est->sent_buf,s->est->sent_size,SESSION_RESPONSE,&s->pack_num_sent);
							rx_session.start_initialized_session(&s->ses);
							s->userInfoSize = rx_session.get_user_info(s->userInfo);
							s->userInfo[s->userInfoSize] = s->userInfo[s->userInfoSize+1] = 0;
							s->flags = SESSION_LAUNCHED;
							s->est->last_size = s->est->rec_size;
							memcpy(s->est->last_buf,s->est->buff,s->est->rec_size);
			                LeaveCriticalSection(&n->cs);
							TCHAR str[400];
							_stprintf_s(str,200,TEXT("Successful Session request from %d.%d.%d.%d:%d, user name: %s, fingerprint %08X %08X %08X %08X %08X %08X %08X %08X"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port,
								s->userInfo,session_id[0],session_id[1],session_id[2],session_id[3],session_id[4],session_id[5],session_id[6],session_id[7]);
							LogMessage(module,str);
							s->est->rec_size = 0;
							s->est->rec_flags = 0;
						}
						else
						{
							s->flags = 0;
							s->ses.reset();
							s->est->tx_session.reset();
							delete s->est;
							s->est = NULL;
			                LeaveCriticalSection(&n->cs);
						// WARNING: someone sent incorrect request
							TCHAR str[400];
							_stprintf_s(str,200,TEXT("Wrong Session request from %d.%d.%d.%d:%d"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port);
							LogMessage(module,str);
						}
						rx_session.reset();
						break;
					}
                }
			    LeaveCriticalSection(&n->cs);
                break;
            case SESSION_RESPONSE:
				s->pack_num_est++;
                if ((s->flags&SESSION_PINGED) && (s->flags&SESSION_RESPONDED) && !(s->flags&SESSION_LAUNCHED) && s->est)
                {
                    if (n->collect_big_buffer(&buffer_coded[NETWORK_ID_LENGTH],pack_len-NETWORK_ID_LENGTH*sizeof(u32),s->est))
                    {
                        size = s->est->tx_session.process_response((u8*)s->est->buff);
                        if ((size > 0) && s->est->tx_session.session_is_active(session_id) && isIdIdentical(s->nd.id,session_id)) // normal response
                        {
                            s->est->tx_session.start_initialized_session(&s->ses);
							s->userInfoSize = s->est->tx_session.get_user_info(s->userInfo);
							s->userInfo[s->userInfoSize] = s->userInfo[s->userInfoSize+1] = 0;
                            s->flags = SESSION_LAUNCHED;
			                LeaveCriticalSection(&n->cs);
							TCHAR str[400];
							_stprintf_s(str,200,TEXT("Successful Session response from %d.%d.%d.%d:%d, user name: %s, fingerprint %08X %08X %08X %08X %08X %08X %08X %08X"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port,
								s->userInfo,session_id[0],session_id[1],session_id[2],session_id[3],session_id[4],session_id[5],session_id[6],session_id[7]);
							LogMessage(module,str);
							break;
                        }
                        else
                        {
                            s->flags = 0;
                            s->ses.reset();
                            s->est->tx_session.reset();
                            delete s->est;
                            s->est = NULL;
			                LeaveCriticalSection(&n->cs);
                            // WARNING: Someone use ID which is not equal to fingerpint
							TCHAR str[400];
							_stprintf_s(str,200,TEXT("Wrong Session response from %d.%d.%d.%d:%d"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port);
							LogMessage(module,str);
							break;
                        }
                    }
                }
                LeaveCriticalSection(&n->cs);
                break;
            case SESSION_PING_REQ:	// just send response on this request
				s->pack_num_est++;
                LeaveCriticalSection(&n->cs);
				CopyMemory(buffer_coded,n->my_id,NETWORK_ID_LENGTH*sizeof(u32));
                buffer_coded[NETWORK_ID_LENGTH] = SESSION_PING_RESP;
				sendto(n->soc,(char*)buffer_coded,(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
				s->pack_num_sent++;
				if (kad) kad->updateNode(&nd,false,NULL);
                break;
            case SESSION_PING_RESP: // answer on our request
				s->pack_num_est++;
                if ((s->flags&SESSION_PINGED) && !(s->flags&SESSION_RESPONDED) && !(s->flags&SESSION_LAUNCHED))
                {
                    s->flags |= SESSION_RESPONDED;
                    if (!s->est) s->est = new ESTABLISHMENT;
                    s->est->rec_flags = 0;
                    s->est->tx_session = n->cmn_tx_session;
                    s->est->rec_size = 0;
                    LeaveCriticalSection(&n->cs);
                    n->send_big_buffer(nd.ip,nd.port,n->sending_buff,n->sending_size,SESSION_REQUEST,&s->pack_num_sent);
					TCHAR str[400];
					_stprintf_s(str,200,TEXT("Again send Session request to %d.%d.%d.%d:%d"),FIRST_IPADDRESS(nd.ip),SECOND_IPADDRESS(nd.ip),THIRD_IPADDRESS(nd.ip),FOURTH_IPADDRESS(nd.ip),nd.port);
					LogMessage(module,str);
                }
                else LeaveCriticalSection(&n->cs);
				if (kad) kad->updateNode(&nd,false,NULL);
                break;
            case SESSION_ENCODED:
			case SESSION_NAME_REQ:
			case SESSION_NAME_RESP:
                if (s->flags & SESSION_LAUNCHED)
                {
                    u32 decr_size = s->ses.decrypt((u8*)buffer_plain,(u8*)&buffer_coded[NETWORK_ID_LENGTH+1],pack_len-(NETWORK_ID_LENGTH+1)*sizeof(u32));
                    s->flags &= ~SESSION_REQ_COUNT_MASK; // clear counter, because responded
                    if (decr_size > sizeof(u32))
                    {
                        if (s->est) // session established normaly, don't need data anymore
                        {
                            s->est->tx_session.reset();
                            delete s->est;
                            s->est = NULL;
                        }
						s->pack_num_sec++;
						if (pack_type == SESSION_ENCODED)
						{
							NET_PACKET np;
							np.buffer = buffer_plain;
							np.buf_len = decr_size;
							np.nd = &nd;
							LeaveCriticalSection(&n->cs);
							parsePacket(&np);
						}
						else if (pack_type == SESSION_NAME_REQ)
						{
							CONNECTION* co;
							if (decr_size >= buffer_plain[0])
							{
								for (co = n->con; co; co = co->next_connection)
								{
									if (!name_compare((u8*)&buffer_plain[1],co->userInfo,min(co->userInfoSize,buffer_plain[0])))
									{
										CopyMemory(buffer_coded,n->my_id,NETWORK_ID_LENGTH*sizeof(u32));
										buffer_coded[NETWORK_ID_LENGTH] = SESSION_NAME_RESP;
										CopyMemory(buffer_plain,co->nd.id,NETWORK_ID_LENGTH*sizeof(u32));
										size = s->ses.encrypt((u8*)&buffer_coded[NETWORK_ID_LENGTH+1],(u8*)buffer_plain,NETWORK_ID_LENGTH*sizeof(u32));
										sendto(n->soc,(char*)buffer_coded,(NETWORK_ID_LENGTH+1)*sizeof(u32)+size,0,(sockaddr*)&addr,sizeof(sockaddr_in));
										break;
									}
								}
							}
							LeaveCriticalSection(&n->cs);
						}
						else
						{
							LeaveCriticalSection(&n->cs);
							if (decr_size >= NETWORK_ID_LENGTH*sizeof(u32))
							{
								u32* id = new u32[NETWORK_ID_LENGTH];
								CopyMemory(id,buffer_plain,NETWORK_ID_LENGTH*sizeof(u32));
								PostMessage(n->mainWnd,WM_USER_FOUND,0,(LPARAM)id);
							}
						}
                    }
                    else
					{
						s->pack_num_drop++;
						LeaveCriticalSection(&n->cs);
					}
                }
                else
				{
					s->pack_num_drop++;
					LeaveCriticalSection(&n->cs);
				}
                break;
            }
        }
	}
	return 0;
}
//---------------------------------------------------
CONNECTION* NETWORK::get_next_node(CONNECTION* cur)
{
    CONNECTION* curc = cur;
    if (cur)
    {
        return cur->next_connection;
    }
    return cur;
}
//---------------------------------------------------
void NETWORK::get_counters(const NODE* nd, u32* est, u32* sec, u32* drop, u32* sent)
{
    EnterCriticalSection(&cs);
    CONNECTION* s = find_connection(nd,false);
	if (s)
	{
		*est = s->pack_num_est;
		*sec = s->pack_num_sec;
		*drop = s->pack_num_drop;
		*sent = s->pack_num_sent;
	}
	else
	{
		*est = 0;
		*sec = 0;
		*drop = 0;
		*sent = 0;
	}
	LeaveCriticalSection(&cs);
}
//---------------------------------------------------
void NETWORK::get_user_info(NODE* nd, LPTSTR info)
{
    EnterCriticalSection(&cs);
    CONNECTION* s = find_connection(nd,false);
	if (s)
	{
		memcpy(info,s->userInfo,s->userInfoSize);
		info[s->userInfoSize/sizeof(TCHAR)]=0;
	}
	else
	{
		cp_strcchcpy(info,user_info_max_length_bytes/sizeof(tchar_t),_T("<unknown>"));
	}
	LeaveCriticalSection(&cs);
}
//---------------------------------------------------
bool NETWORK::findUserByName(u8* userInfo, u16 size, u32* id)
{
	CONNECTION *s;
	u32 tx_buffer_plain[200];
	u32 tx_buffer_coded[220];
	sockaddr_in addr = {0};
	//
	if (size > user_info_max_length_bytes) return false;
    EnterCriticalSection(&cs);
	// try to find locally
	for (s = con; s; s = s->next_connection)
	{
		if (!name_compare(userInfo,s->userInfo,min(s->userInfoSize,size)))
		{
			u32* id_msg = new u32[NETWORK_ID_LENGTH];
			memcpy(id,s->nd.id,sizeof(u32)*NETWORK_ID_LENGTH);
			memcpy(id_msg,s->nd.id,sizeof(u32)*NETWORK_ID_LENGTH);
			LeaveCriticalSection(&cs);
			PostMessage(mainWnd,WM_USER_FOUND,0,(LPARAM)id);
			return true;
		}
	}
	// try to find externally
	CopyMemory(tx_buffer_coded,my_id,NETWORK_ID_LENGTH*sizeof(u32));
	for (s = con; s; s = s->next_connection)
	{
		if(s->flags&SESSION_LAUNCHED)
		{
			addr.sin_family = AF_INET;
			addr.sin_addr.s_addr = htonl(s->nd.ip);
			addr.sin_port = htons(s->nd.port);
			tx_buffer_plain[0] = size;
			CopyMemory(&tx_buffer_plain[1], userInfo, size);
			tx_buffer_coded[NETWORK_ID_LENGTH] = SESSION_NAME_REQ;
			size = s->ses.encrypt((u8*)&tx_buffer_coded[NETWORK_ID_LENGTH+1],(u8*)tx_buffer_plain,((size+3)/4)*4+sizeof(u32));
			size = (u32)sendto(soc,(char*)tx_buffer_coded,size+(NETWORK_ID_LENGTH+1)*sizeof(u32),0,(sockaddr*)&addr,sizeof(sockaddr_in));
			s->pack_num_sent++;
			s->flags++;
			if ((s->flags&SESSION_REQ_COUNT_MASK) >= SESSION_REQ_COUNT_MAX) // so long time no any answer. Reset session
			{
				s->flags = 0;
				s->ses.reset();
			}
		}	
	}
	LeaveCriticalSection(&cs);
	return false;
}
//---------------------------------------------------
bool NETWORK::is_user_online(const u32* id)
{
	CONNECTION *s;
	bool res = false;
	//
    EnterCriticalSection(&cs);
	for (s = con; s; s = s->next_connection)
	{
		if (isIdIdentical(id,s->nd.id))
		{
			if (s->flags&SESSION_LAUNCHED) res = true;
			else res = false;
			break;
		}
	}
	LeaveCriticalSection(&cs);
	return res;
}
//---------------------------------------------------
bool NETWORK::findNode(const u32 *id, NODE *nd)
{
	CONNECTION *s;
	bool res = false;
	//
    EnterCriticalSection(&cs);
	for (s = con; s; s = s->next_connection)
	{
		if (isIdIdentical(id,s->nd.id))
		{
			*nd = s->nd;
			res = true;
			break;
		}
	}
	LeaveCriticalSection(&cs);
	return res;
}
//---------------------------------------------------
void NETWORK::update_ip(NODE* nd)
{
	CONNECTION *s;
	bool res = false;
	//
    EnterCriticalSection(&cs);
	for (s = con; s; s = s->next_connection)
	{
		if (isIdIdentical(nd->id,s->nd.id))
		{
			nd->ip = s->nd.ip;
			nd->port = s->nd.port;
			break;
		}
	}
	LeaveCriticalSection(&cs);
}
//---------------------------------------------------
void NETWORK::reset_sessions(void)
{
	CONNECTION *s;
    u8 passw[password_max_length];
	u32 pas_len = getPassword(passw);
	if (pas_len)
	{
		sending_size = cmn_tx_session.init_session((u8*)sending_buff, getSecretData(), passw, pas_len);
	}
    EnterCriticalSection(&cs);
	for (s = con; s; s = s->next_connection)
	{
		s->flags = 0;
		s->ses.reset();
	}
	LeaveCriticalSection(&cs);
}
//---------------------------------------------------
