#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <Shlwapi.h>
#include <Windns.h>

#include "resource.h"
#include "settings.h"
#include "upnp.h"
#include "log.h"
#include "kademlia.h"
#include "main.h"
#include "password.h"
//----------------------------------------------------------------------------
void SelectFile(HWND hWnd, UINT idc)
{
	TCHAR FileName[MAX_PATH];
	OPENFILENAME File = {0};
	GetDlgItemText(hWnd,idc,FileName,MAX_PATH);
	File.lStructSize = sizeof(OPENFILENAME);
	File.hwndOwner = hWnd;
	File.lpstrFile = FileName;
	File.nMaxFile = MAX_PATH;
	File.lpstrFilter = TEXT("ZAS files (*.zas)\0*.zas\0*.*\0*.*\0");
	File.nFilterIndex = 1;
	File.Flags = OFN_PATHMUSTEXIST;
   	if (!GetOpenFileName(&File))
   		return;
	SetDlgItemText(hWnd,idc,FileName);
}
//----------------------------------------------------------------------------
static void SetFileInit(HWND hWnd, settings_t* set)
{
	SetProp(hWnd,TEXT("set"),set);
	SetDlgItemText(hWnd,IDC_SET_KEYFILE,set->keyFileName);
	SetDlgItemText(hWnd,IDC_SET_NETFILE,set->userFileName);
	SetDlgItemText(hWnd,IDC_SET_FGPFILE,set->fgpFileName);
	SetDlgItemText(hWnd,IDC_SET_LIKESFILE,set->likesFileName);
	SetDlgItemText(hWnd,IDC_SET_LOGFILE,set->logFileName);
	CheckDlgButton(hWnd,IDC_LOG_ENABLE,set->enableLogFile?BST_CHECKED:BST_UNCHECKED);
	SetDlgItemInt(hWnd,IDC_SET_REMEMBER,set->passwordTime,FALSE);
}
//----------------------------------------------------------------------------
static void SetFileDestroy(HWND hWnd)
{
	settings_t* set = (settings_t*)RemoveProp(hWnd,TEXT("set"));
	GetDlgItemText(hWnd,IDC_SET_KEYFILE,set->keyFileName,MAX_PATH);
	GetDlgItemText(hWnd,IDC_SET_NETFILE,set->userFileName,MAX_PATH);
	GetDlgItemText(hWnd,IDC_SET_FGPFILE,set->fgpFileName,MAX_PATH);
	GetDlgItemText(hWnd,IDC_SET_LIKESFILE,set->likesFileName,MAX_PATH);
	GetDlgItemText(hWnd,IDC_SET_LOGFILE,set->logFileName,MAX_PATH);
	set->enableLogFile = (BST_CHECKED == IsDlgButtonChecked(hWnd,IDC_LOG_ENABLE));
	set->passwordTime = GetDlgItemInt(hWnd,IDC_SET_REMEMBER,NULL,FALSE);
}
//----------------------------------------------------------------------------
static INT_PTR CALLBACK SetFileProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		SetFileInit(hWnd,(settings_t*)((PROPSHEETPAGE*)lParam)->lParam);
		return TRUE;
		//
    case WM_DESTROY:
		SetFileDestroy(hWnd);
		return 0;
		//
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_SET_KEYFILESEL:
		case IDC_SET_NETFILESEL:
		case IDC_SET_FGPFILESEL:
		case IDC_SET_LIKESFILESEL:
		case IDC_SET_LOGFILESEL:
			SelectFile(hWnd,LOWORD(wParam)-1);
			return TRUE;
		}
		return FALSE;
	}
	return FALSE;
}
//----------------------------------------------------------------------------/----------------------------------------------------------------------------
static void SetNetworkInit(HWND hWnd, settings_t* set)
{
	SetProp(hWnd,TEXT("set"),set);
	SetDlgItemInt(hWnd,IDC_SET_LOCPORT,set->localPort,FALSE);
	SetDlgItemInt(hWnd,IDC_SET_REMPORT,set->remotePort,FALSE);
	CheckDlgButton(hWnd,IDC_UPNP_ENABLE,set->enableUPnP?BST_CHECKED:BST_UNCHECKED);
}
//----------------------------------------------------------------------------
static void SetNetworkDestroy(HWND hWnd)
{
	settings_t* set = (settings_t*)RemoveProp(hWnd,TEXT("set"));
	set->localPort = GetDlgItemInt(hWnd,IDC_SET_LOCPORT,NULL,FALSE);
	set->remotePort = GetDlgItemInt(hWnd,IDC_SET_REMPORT,NULL,FALSE);
	set->enableUPnP = (BST_CHECKED==IsDlgButtonChecked(hWnd,IDC_UPNP_ENABLE));
}
//----------------------------------------------------------------------------
static INT_PTR CALLBACK SetNetworkProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		SetNetworkInit(hWnd,(settings_t*)((PROPSHEETPAGE*)lParam)->lParam);
		return TRUE;
		//
    case WM_DESTROY:
		SetNetworkDestroy(hWnd);
		return 0;
		//
	}
	return FALSE;
}
//----------------------------------------------------------------------------
#define PAGES_NUM	2
//----------------------------------------------------------------------------
void SettingDialog(HWND hWnd, settings_t* set, USERS* usr, KADEMLIA* kad)
{
	PROPSHEETPAGE psp[PAGES_NUM];
	PROPSHEETHEADER psh;
	settings_t loc_setttings = *set;
	
	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_USETITLE;
	psp[0].hInstance = GetModuleHandle(NULL);
	psp[0].pszTemplate = TEXT("SettingsFiles");
	psp[0].pfnDlgProc = SetFileProc;
	psp[0].pszTitle = TEXT("File");
	psp[0].lParam = (LPARAM)&loc_setttings;
	psp[0].pfnCallback = NULL;
	psp[0].pszIcon = NULL;
	//
	psp[1].dwSize = sizeof(PROPSHEETPAGE);
	psp[1].dwFlags = PSP_USETITLE;
	psp[1].hInstance = GetModuleHandle(NULL);
	psp[1].pszTemplate = TEXT("SettingsNetwork");
	psp[1].pfnDlgProc = SetNetworkProc;
	psp[1].pszTitle = TEXT("Network");
	psp[1].lParam = (LPARAM)&loc_setttings;
	psp[1].pfnCallback = NULL;
	psp[1].pszIcon = NULL;
	//
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOCONTEXTHELP | PSH_NOAPPLYNOW;
	psh.hwndParent = hWnd;
	psh.hInstance = GetModuleHandle(NULL);
	psh.pszCaption = TEXT("Settings");
	psh.nPages = PAGES_NUM;
	psh.nStartPage = 0;
	psh.ppsp = (LPCPROPSHEETPAGE) &psp;
	psh.pfnCallback = NULL;
	if(PropertySheet(&psh)) // copy settings if update is successful
	{
		if (set->enableUPnP^loc_setttings.enableUPnP)
		{
			if (loc_setttings.enableUPnP)
			{
				if (!setupUPnP(loc_setttings.localPort,loc_setttings.remotePort,0))
				{
					MessageBox(hWnd,TEXT("Can't setup UPnP"),TEXT("Warning"),MB_OK|MB_ICONWARNING);
				}
			}
			else
			{
				if (!setupUPnP(set->localPort,set->remotePort,1))
				{
					MessageBox(hWnd,TEXT("Can't disable UPnP"),TEXT("Warning"),MB_OK|MB_ICONWARNING);
				}
			}
		}
		if (_tcscmp(set->keyFileName,loc_setttings.keyFileName))
		{
			LoadKey(hWnd, loc_setttings.keyFileName, getSecretData());
		}
		if (_tcscmp(set->logFileName,loc_setttings.logFileName) || (set->enableLogFile^loc_setttings.enableLogFile))
		{
			SendMessage(g_logWnd,WM_APPLY_LOG_SETTINGS,0,(LPARAM)&loc_setttings);
		}
		if (_tcscmp(set->userFileName,loc_setttings.userFileName))
		{
			usr->load_nodes(kad,loc_setttings.userFileName);
		}
		*set = loc_setttings;
	}
}
//-----------------------------------------------------------------------
static void getSettingsFileName(LPTSTR fileName)
{
	GetModuleFileName(NULL,fileName,MAX_PATH);
	CopyMemory(&fileName[_tcslen(fileName)-3],TEXT("set"),3*sizeof(TCHAR));
}
//-----------------------------------------------------------------------
void LoadSettings(HWND hWnd, settings_t* set)
{
	TCHAR fileName[MAX_PATH];
	getSettingsFileName(fileName);
	if (!PathFileExists(fileName))
	{
		TCHAR msg[MAX_PATH+100];
		_stprintf_s(msg,MAX_PATH+100,TEXT("Can't find settings file:\n%s\nDefault settings will be used"),fileName);
		MessageBox(hWnd,msg,TEXT("Warning"),MB_OK|MB_ICONWARNING);
	}
	GetPrivateProfileString(TEXT("Files"),TEXT("Key"),TEXT("key.zas"),set->keyFileName,MAX_PATH,fileName);
	GetPrivateProfileString(TEXT("Files"),TEXT("Users"),TEXT("users.xml"),set->userFileName,MAX_PATH,fileName);
	GetPrivateProfileString(TEXT("Files"),TEXT("Fingerprints"),TEXT("fingerprints.zas"),set->fgpFileName,MAX_PATH,fileName);
	GetPrivateProfileString(TEXT("Files"),TEXT("Likes"),TEXT("likes.xml"),set->likesFileName,MAX_PATH,fileName);
	GetPrivateProfileString(TEXT("Files"),TEXT("Log"),TEXT("log.txt"),set->logFileName,MAX_PATH,fileName);
	set->enableLogFile= GetPrivateProfileInt(TEXT("Files"),TEXT("EnableLog"),0,fileName);

	set->passwordTime = GetPrivateProfileInt(TEXT("Password"),TEXT("Time"),120000,fileName);

	set->localPort = GetPrivateProfileInt(TEXT("Network"),TEXT("LocalPort"),31265,fileName);
	set->remotePort = GetPrivateProfileInt(TEXT("Network"),TEXT("RemotePort"),31265,fileName);
	set->enableUPnP = GetPrivateProfileInt(TEXT("Network"),TEXT("EnableUPnP"),0,fileName);
}
//-----------------------------------------------------------------------
void SaveSettings(HWND hWnd, settings_t* set)
{
	TCHAR val[100];
	TCHAR fileName[MAX_PATH];
	getSettingsFileName(fileName);
	WritePrivateProfileString(TEXT("Files"),TEXT("Key"),set->keyFileName,fileName);
	WritePrivateProfileString(TEXT("Files"),TEXT("Users"),set->userFileName,fileName);
	WritePrivateProfileString(TEXT("Files"),TEXT("Fingerprints"),set->fgpFileName,fileName);
	WritePrivateProfileString(TEXT("Files"),TEXT("Likes"),set->likesFileName,fileName);
	WritePrivateProfileString(TEXT("Files"),TEXT("Log"),set->logFileName,fileName);
	_stprintf_s(val,100,TEXT("%d"),set->enableLogFile);
	WritePrivateProfileString(TEXT("Files"),TEXT("EnableLog"),val,fileName);

	_stprintf_s(val,100,TEXT("%d"),set->passwordTime);
	WritePrivateProfileString(TEXT("Password"),TEXT("Time"),val,fileName);

	_stprintf_s(val,100,TEXT("%d"),set->localPort);
	WritePrivateProfileString(TEXT("Network"),TEXT("LocalPort"),val,fileName);
	_stprintf_s(val,100,TEXT("%d"),set->remotePort);
	WritePrivateProfileString(TEXT("Network"),TEXT("RemotePort"),val,fileName);
	_stprintf_s(val,100,TEXT("%d"),set->enableUPnP);
	WritePrivateProfileString(TEXT("Network"),TEXT("EnableUPnP"),val,fileName);
}
//-----------------------------------------------------------------------
VOID URL2IP(LPCTSTR url, u32* ip)
{
	PDNS_RECORD pDnsRecord;
	*ip = 0;
	if (DNS_RCODE_NOERROR == DnsQuery(url,DNS_TYPE_A,DNS_QUERY_NO_NETBT|DNS_QUERY_NO_HOSTS_FILE,NULL,&pDnsRecord,NULL))
	{
		for (PDNS_RECORD rr = pDnsRecord; rr; rr = rr->pNext)
		{
			if (rr->wType == DNS_TYPE_A)
			{
				*ip = ntohl(rr->Data.A.IpAddress);
				break;
			}
		}
		DnsRecordListFree(pDnsRecord,DnsFreeRecordList);
	}
}
//-----------------------------------------------------------------------
